/*
 *  GenericViewer.h
 *
 *  Generic Viewer class derived from Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


//ruckelt wenn caminfb active ist lt sich vielleicht nicht verhindern

#ifndef GENERICVIEWER_H
#define GENERICVIEWER_H

#include "Viewer.h"
#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/toolbar.h>
#include "widgets/Gtk_CameraControl.h"
#include "widgets/Gtk_LightControl.h"
#include "Vector3.h"


/** Generic Viewer.
    Derived from Viewer class. It is supposed to have some additional functions
    which are still kept rather general so that this Class can be used in many
    applications. At the moment the Generic Viewer contains a toolbox, which
    is inserted in Viewer::upper_hbox, a ViewerToolbox member object, and a 
    CameraControl member object which supports an additional mouse mode.
*/

class GenericViewer: public Viewer
{

  /* some variables for the mouse control */

  gdouble mouse_start_x, mouse_start_y;
  //problem: event press and motion notify not in right order
  //...need this flag to be sure press was recognized
  gboolean mouse1_pressed; 
  gboolean mouse2_pressed;
  gboolean mouse3_pressed;


  /** stores the current mouse mode */
  int mouse_mode;

  /** change the current mode; will be connected to caminfb.mode_set signal */
  void set_mouse_mode(int mode) { mouse_mode = mode;} 

  /** Set light position. Used to get new light position from LightControl */
  void set_light(); 

  /** The entry changes of the CameraControl window will be handled here, namely
      by changing the camera stats. */
  void entry_activated(int id); 

 public:

  /** Constructor. Argument is a string which will label the window. */
  GenericViewer(char* label);

  /** the virtual mouse handler methods are overriden to realize the new control
      mode. If the old mode is active, the underlying Viewer methods are called.*/
  virtual gint gl_motion_notify_handler_imp(GdkEventMotion* event); 

  /** the virtual mouse handler methods are overriden to realize the new control
      mode. If the old mode is active, the underlying Viewer methods are called. */
  virtual gint gl_button_press_handler_imp(GdkEventButton* event); 

  /** the virtual mouse handler methods are overriden to realize the new control
      mode. If the old mode is active, the underlying Viewer methods are called.  */
  virtual gint gl_button_release_handler_imp(GdkEventButton* event); 


  /** The CameraControl window. Gives access to camera settings. */
  CameraControl camera_control; 

  /** The LightControl window. Allows to change lighting.*/
  LightControl light_control; 

  /** This signal is connected to the gl_changed_event of the Viewer. Whenever 
      the camera stats change (eg. by Viewers mouse control), the CameraControl 
      window will be updated with the new coordinates. */
  void update_camera_control(); 

  /** Handles the buttons of the ViewerToolbox member */
  void tool_handler(int id); 

  /** The GenericViewers toolbar is public so new tools can be inserted by 
      derived Classes for example (might change to protected though).  */
  Gtk::Toolbar toolbar; 

  /** frame for the GV tools */
  Gtk::Frame gv_toolbar_frame;
  /** bar for the GV tools */
  Gtk::Toolbar gv_toolbar; 

};















#endif
