/*
 *  QuatFractCreator.cc
 *
 *  The center of your fractal world
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractCreator.h"
#include "QuatMath.h"
#include "OpenGLUtils.h"
#include "Cube.h"


#include <vector>
#include <list>
#include <map>
#include <string>

//TODO shall be removed (capsulating from opengl)
#include <GL/gl.h> 
#include <GL/glu.h> 
#include <GL/glut.h> 



void QuatFractCreator::set_func(iter_func m) { 
  fparams.iterFunc = m; 
  switch(m){
  case SQR:
    konv = &QuatFractCreator::iter_sqr;
    break;
  case CUBIC:
    konv = &QuatFractCreator::iter_cubic;
    break;
  }
}


void QuatFractCreator::set_fparams(const FractParams& fp) {
  this->fparams = fp;
  set_func(fparams.iterFunc);
}

/* tests if the function converges at q */
bool QuatFractCreator::iter_sqr(Quat &q) {
//   double *a = q.get_vec();

//   if ( a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3] < 0.5)
//     return true;
//   else return false;
  


  for(int i = 0; i < fparams.maxiter; i++) {
    //q.g(fparams.C);
    q.f(fparams.C);
    if( q.abs() > fparams.bailout)
      return false;
  }

  return true;
}


bool QuatFractCreator::iter_cubic(Quat &q) {
//   double *a = q.get_vec();

//   if ( a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3] < 0.5)
//     return true;
//   else return false;
  


  for(int i = 0; i < fparams.maxiter; i++) {
    q.g(fparams.C);
    //q.f(fparams.C);
    if( q.abs() > fparams.bailout)
      return false;
  }

  return true;
}


QuatFractCreator::QuatFractCreator( double a, double b, double c, double d) 
  : poly_map(256), normal_map(256), konv( &QuatFractCreator::iter_sqr){

  fparams.polyMethod = CUBE;
  
  fparams.xMin = -2;
  fparams.yMin = -1;
  fparams.zMin = -1;
  fparams.xMax = 2;
  fparams.yMax = 1;
  fparams.zMax = 1;

  fparams.step = 0.05;

  fparams.bailout = 40;
  fparams.maxiter = 10;

  fparams.C = Quat( a, b, c, d);
  fparams.x4 = 0.0;

  set_func(SQR);
}



//don't look
void QuatFractCreator::register_data (SigC::ScopeNode *)
{
 
}





