/*
 *  QuatFractCreator_Cube.cc
 *
 *  Cube Polygonization Algorithm
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractCreator.h"
#include "QuatMath.h"
#include "OpenGLUtils.h"
#include "Cube.h"

#include <vector>
#include <list>
#include <map>
#include <string>

//shall be removed (capsulating from opengl)

#include <GL/gl.h> 
#include <GL/glu.h> 
#include <GL/glut.h> 


#define COLORIZE_POLYS 0
#define DEBUG 0


/* this is the brute force method: just iterate through 
   all cells and check if we are on the surface or not
   , if we are render a polygon.
   I use the cube cache and the conv cache that have to be initialized before.   
 */
void QuatFractCreator::cube_polygonize() {

  /* counts the different variants of polygons within the cubes */
//   vector<int> cubevarcounts(256);
//   for( int i = 0; i < 256; i++)
//     cubevarcounts[i] = 0;


  /* polygon index counter */
  int pol_i = 0;

  /* clear the point_to maps and the cube vector*/
  point_to_polys.clear();
  point_to_normal.clear();
  cube_v.clear();


  /* the strides that have to be used to access the right elements in the
     conv_map which is 1-dimensional and has to be accesed like
     conv_map(i*i_stride + j*j_stride + k) */
  unsigned int j_stride = (int) ceil( (fparams.zMax - fparams.zMin)/fparams.step );
  unsigned int i_stride = (int) ceil( (fparams.yMax - fparams.yMin)/fparams.step ) * j_stride;

  /* clear the convergence cache */
  conv_map.clear();
  conv_map.insert(conv_map.end(), 
		  (int) ceil( (fparams.xMax - fparams.xMin)/fparams.step ) * i_stride,
		  0);


  cout << "cache size: " << conv_map.size() << endl;
  cout << "i_stride: " << i_stride << endl;
  cout << "j_stride: " << j_stride << endl;


  cout << "rendering polygons";

  /* setup the vectors pointing to the edges (static cube members) */
  Cube::init_edge_vec( fparams.step);


  /* buffer for setting up the vectors in the cubes */
  vector<double> vect(3);

  /* iterator pointing at the map of the current vertex in conv_map */
  vector< unsigned char>::iterator ci = conv_map.begin(), ci_buf;

  /* the cut into 4-dim quat-space is done by setting
     x4 = fparams.x4, this is the most efficient and least general way
     will have to find a fast algo doing that with every
     hyperplane */
  

  for( double a = fparams.xMin; a <= fparams.xMax - fparams.step; a += fparams.step) {
    for( double b = fparams.yMin; b <= fparams.yMax - fparams.step; b += fparams.step) {
      for( double c = fparams.zMin; c <= fparams.zMax - fparams.step; c += fparams.step){
	/* a cube is created and the vertices are set up */
	unsigned char bitm = 0;
	int r = 0;
	Cube cube(8, a, b, c);
	for( int i = 0; i < 2; i++) 
	  for( int j = 0; j < 2; j++) 
	    for( int k = 0; k < 2; k++) {
	      
	      /* look in conv map if it converges or not */
	      ci_buf = ci + i*i_stride + j*j_stride + k;
	      if( (*ci_buf) == 0x2) 
		cube.in[r] = false;
	      else if( (*ci_buf) == 0x3) {
		cube.in_count++;
		cube.in[r] = true;
	      } else {
		Quat q(a + i*fparams.step, b + j*fparams.step, c + k*fparams.step, fparams.x4);
		*ci_buf = (this->*konv)(q) ? 0x3 : 0x2;
		if( (*ci_buf) == 0x2) 
		  cube.in[r] = false;
		else if( (*ci_buf) == 0x3) {
		  cube.in_count++;
		  cube.in[r] = true;
		}
	      }
	      
	      /* now we rip out the char bitstring used to cache the 
		 cube polygons */
	      bitm <<= 1;
	      bitm |= cube.in[r];
	      
	      r++;
	    }
	
	
	/* count variants of polygons */
	//cubevarcounts[bitm]++;
	
	/* if we are at the surface we save the cube in the
	   cube_v vector and add it to the map for each vertex 
	   involved */
	if( cube.in_count > 0 && cube.in_count < 8) {
	  cube.polygonize( poly_map[bitm], normal_map[bitm]);
	  cube_v.push_back( cube);
	  for( list<vector<double> >::iterator pv = cube.pol.begin(); 
	       pv != cube.pol.end(); pv++) 
	    point_to_polys[*pv].push_back(pol_i);
	  
	  pol_i++;
	}
	ci++;
      }
      /* this has to be done to prevent the cubes at the limit of 
	 the bounding box to be initialized ... */
      ci++;
    }
    /* ... and this, too */
    ci += j_stride;

    render_progress( (a - fparams.xMin) / (fparams.xMax - fparams.xMin));

  }

  cout << "done" << endl;
  cout << "polygon count: " << pol_i << endl;
//   cout << "variants: " << endl;
//   for( int i = 0; i< 256; i++) 
//     cout << i << "\t-->\t" << cubevarcounts[i] << endl;
  cout.flush();

}




/* merges the normals of the cubes in cube_v and finally draws to 
   the OpenGL context 
*/
void QuatFractCreator::cube_merge_n_draw() {
  
  cout << "merging normals and drawing.";
  cout.flush();

  /* finally iterate through polygons, merge normals at each vertex
     and draw the surface */
  for( vector< Cube>::iterator ci = cube_v.begin(); ci != cube_v.end(); ci++) {

    glBegin(GL_POLYGON);

    glColor3f(1, 1, 1);
    /* visualize the different types of cube polygons 
       with colors */
    if(COLORIZE_POLYS) {
      glColor3f(1, 1, 1);
      switch( ci->pol.size()) {
      case 4: glColor3f(0, 0, 1);
	break;
      case 5: glColor3f(0, 1, 0);
	break;
      case 6: glColor3f(1, 0, 0);
      }
    }

    /* iterate through vertices in polygon *ci */
    for( list< vector<double> >::iterator vi = ci->pol.begin(); 
	 vi != ci->pol.end(); vi++) {

      vector< double> normal(3);

      /* is *vi already cached? */
      if( point_to_normal[*vi].empty()) {
	for(int i = 0; i < 3; i++)
	  normal[i] = 0;
	
	/* find all polygons including vertex *vi and 
	   merge normals */
	list< int> l = point_to_polys[*vi];
	for( list< int>::iterator pi = l.begin(); pi != l.end(); pi++) {
	  for(int i = 0; i < 3; i++)
	    normal[i] += cube_v[*pi].norm[i];
	}
	double nabs = sqrt(normal[0]*normal[0]
			   +normal[1]*normal[1]
			   +normal[2]*normal[2]);
	
	for(int i = 0; i < 3; i++)
	  normal[i] /= nabs;

	/* we will map *vi its normal in order to eliminate 
	   unnecessary draw operations */
	point_to_normal[*vi] = normal;

      } 
      /* *vi is cached, so we'll take the cached normal */
      else {
	normal = point_to_normal[*vi];
      }

	
      /* now we have the combined normal and are able to 
	 set the vertex */
      glNormal3f( normal[0], normal[1], normal[2]);
      glVertex3f( (*vi)[0], (*vi)[1], (*vi)[2]);


    }
    glEnd();
  }
  cout << "done" << endl;
  cout.flush();
}


void cube_test(){};


/* stores all 2^8 = 256 possible cube states 
   AND the normals... 
*/
void QuatFractCreator::cube_init() {
  
  cout << "initializing cube cache";
  cout.flush();

  /* setup the vectors pointing to the edges (static cube members) */
  Cube::init_edge_vec( fparams.step);
  /* ... and our cube */
  Cube cube(8, 0, 0, 0);


  /* here we iterate through all possible Cube configurations
     and cache 
     - the polygon (meaning the list of indices to the vectors
       pointing on the involved edges -> static edge_vec)
     - the normal of this polygon (which is invariant under translation)
  */
  int i[8];
  unsigned char bitm;
  for( i[0] = 0; i[0] < 2; i[0]++) {
    for( i[1] = 0; i[1] < 2; i[1]++) 
      for( i[2] = 0; i[2] < 2; i[2]++) 
	for( i[3] = 0; i[3] < 2; i[3]++) 
	  for( i[4] = 0; i[4] < 2; i[4]++) 
	    for( i[5] = 0; i[5] < 2; i[5]++) 
	      for( i[6] = 0; i[6] < 2; i[6]++) 
		for( i[7] = 0; i[7] < 2; i[7]++) {
		  bitm = 0;
		  cube.in_count = 0;
		  for( int j = 0; j < 8; j++) {
		    cube.in[j] = i[j];
		    if(DEBUG) cout << (int) i[j];
		    cube.in_count += i[j];

		    /* the i-th bit of our map says if the 
		       i-th cube corner is in or not*/
		    bitm <<= 1;
		    bitm |= i[j];
		  }

		  /* the polygon index list is saved for this bitmap*/
		  poly_map[bitm] = cube.get_poly_index_list();
		  
		  /* ... and the normal, too */
		  normal_map[bitm] = cube.get_normal(poly_map[bitm]);
		  
		  /* debug output */
		  if(DEBUG) {
		    cout << " = " << (int) bitm << ":  ";  
		    for( list<unsigned char>::iterator li = poly_map[bitm]->begin();
			 li != poly_map[bitm]->end();
			 li++) {
		      cout << (int) *li << ",";
		    }
		    cout << endl;
		  }
		  
		}
    cout << ".";
    cout.flush();
  }
  cout << "done" << endl << flush;
}




//Done on the fly now

/* calculating the convergence map (conv_map) using
   the actual parameters
*/
// void QuatFractCreator::init_conv_map() {

//   cout << "initializing conv map";
//   cout.flush();

//   /* clear the convergence cache */
//   conv_map.clear();

//   /* the cut into 4-dim quat-space is done by brutely setting
//      x4 = 0, this is the most efficient and least general way
//      will have to find a fast algo doing that with every
//      hyperplane */
//   for( double a = fparams.xMin; a < fparams.xMax; a += fparams.step) {
//     for( double b = fparams.yMin; b < fparams.yMax; b += fparams.step) {
//       for( double c = fparams.zMin; c < fparams.zMax; c += fparams.step){

// 	/* 7th bit = valid tag / 8th bit = convergence, 
// 	   the others could be used for coloring... */
// 	Quat q(a, b, c, 0);
// 	conv_map.push_back( (this->*konv)(q) ? 0x3 : 0x2);
//       }
//     }
	
//     cout << ".";
//     cout.flush();
//   }

//   cout << "done" << endl;
// }
  
  

