/*
 *  QuatFractCreator_Track.cc
 *
 *  Surface Tracking Polygonization Algorithm
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractCreator.h"
#include "QuatMath.h"
#include "OpenGLUtils.h"
#include "Cube.h"
#include "tetra_Cube.h"


#include <vector>
#include <list>
#include <map>
#include <string>

//shall be removed (capsulating from opengl)

#include <GL/gl.h> 
#include <GL/glu.h> 
#include <GL/glut.h> 



/* surface tracking system: we randomly pick cells until we
   hit the surface. Then track along the surface and render
   polygons with interpolated normals.  */

void QuatFractCreator::track_polygonize() {
  double min = -2;
  double max = 2;
  double step = 0.1;
  int step_nr = int( (max - min) / step);
  bool hit_surface = 0;

  Quat *x = new Quat();
  vector<double> vect(3);

  /* this is the area in which we are rendering */
  glColor3f(0, 1.0, 0);
  glutWireCube (2*max);


  /* visualize the cell size */
  glTranslatef(2.0, 2.0, 2.0);
  glColor3f(1.0, 0, 0);
  glutWireCube (step);
  glTranslatef(-2.0, -2.0, -2.0);

  glColor3f(1.0, 1.0, 1.0);
  
  Cube surface_cube(8);

  /* randomly pick cells */
  while( !hit_surface) {
    double a = int((double(rand())/RAND_MAX)*step_nr)*step + min;
    double b = int((double(rand())/RAND_MAX)*step_nr)*step + min;
    double c = int((double(rand())/RAND_MAX)*step_nr)*step + min;
    int r = 0;
    Cube cube(8);
    for( int i = 0; i < 2; i++) 
      for( int j = 0; j < 2; j++) 
	for( int k = 0; k < 2; k++) {
	  vect[0] = a+i*step;
	  vect[1] = b+j*step;
	  vect[2] = c+k*step;
	  cube.v[r] = vect;
	  *x = Quat(vect[0], vect[1], vect[2], 0);
	  if( konv(*x) ) {
	    cube.in_count++;
	    cube.in[r] = true;
	  } else {
	    cube.in[r] = false;
	  }
	  r++;
	}
    cout << ".";
    cout.flush();
    if( (cube.in_count > 0) && (cube.in_count < 8)) {
      surface_cube = cube;
      hit_surface = true;
      cout << "hit at (" << a << "," << b << "," << c << ")" << endl;
      cout.flush();
      glTranslatef(a, b, c);
      glColor3f(1.0, 0, 0);
      glutWireCube (step);
      glTranslatef(-a, -b, -c);
      glColor3f(1.0, 1, 1);

      cube.polygonize();


    }
  }
}



