/*
 *  QuatFractCreator_Vrend.cc
 *
 *  Vrend data format output
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractCreator.h"
#include "QuatMath.h"
#include "Cube.h"

#include <vector>
#include <list>
#include <map>
#include <string>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


/* write vrend data of current fractal as defined by FractParams
*/
void QuatFractCreator::write_vrend( char* outfile) {

  /* this is C code (fix to C++) */
  int outfd;
  int ii = 0;
  unsigned int x_size = (int) ceil( (fparams.xMax - fparams.xMin)/fparams.step );
  unsigned int y_size = (int) ceil( (fparams.yMax - fparams.yMin)/fparams.step );
  unsigned int z_size = (int) ceil( (fparams.zMax - fparams.zMin)/fparams.step );
  int one = 1;

  cout << x_size << "," << y_size << "," << z_size << endl;

  unsigned char buf;
  char *header = (char*) calloc(24, 1);
  char *data = (char*) malloc( x_size*y_size*z_size);
  char *begin_data = data;

  if ( (outfd = open( outfile, O_WRONLY|O_CREAT|O_TRUNC, 00644)) == -1) {
    cerr << "error: could not open " << outfile << endl;
    exit(1);
  }

  cout << "writing header" << flush;

  memcpy (header + 3, (const void *) &x_size, 1);
  memcpy (header + 7, (const void *) &y_size, 1);
  memcpy (header + 11, (const void *) &z_size, 1);

  memcpy( header + 15, (const void *) &one, 1);
  memcpy( header + 19, (const void *) &one, 1);
  memcpy( header + 23, (const void *) &one, 1);

  if ( ! write(outfd, header, 24) ) {
    cerr << "error: could not write header into " << outfile << endl;
    exit(1);
  }

  cout << "." << endl << flush;

  
  cout << "writing data" << flush;
  /* the cut into 4-dim quat-space is done by brutely setting
     x4 = 0, this is the most efficient and least general way
     will have to find a fast algo doing that with every
     hyperplane */
  for( double c = fparams.zMin; c < fparams.zMax; c += fparams.step){
    for( double b = fparams.yMin; b < fparams.yMax; b += fparams.step) {
      for( double a = fparams.xMin; a < fparams.xMax; a += fparams.step) {
	/* 7th bit = valid tag / 8th bit = convergence, 
	   the others could be used for coloring... */
	Quat q(a, b, c, fparams.x4);
	buf = (this->*konv)(q) ? 0x80 : 0x00;
	memcpy( data++, (const void*) &buf, 1);
      }
    }    
    cout << ".";
    cout.flush();
  }
  
  if ( ! write(outfd, begin_data, x_size*y_size*z_size) ) {
    cerr << "error: could not write data into " << outfile << endl;
    exit(1);
  }

  close(outfd);
  cout << "done" << endl;
}
