/*
 *  QuatFractViewer.cc
 *
 *  Fractal Viewer derived from Generic Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractViewer.h"

#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <gtk--/main.h>
#include <gtk--/toolbar.h>
#include <gtk--/button.h>
#include <gtk--/togglebutton.h>
#include <gtk--/radiobutton.h>
#include <gtk--/box.h>
#include <gtk--/window.h>


#define THREADING 0

using SigC::bind;
using SigC::slot;

bool QuatFractViewer::finish_render = true;


/* static default qf ? */
QuatFractViewer::QuatFractViewer():
  GenericViewer("Fractal Viewer"), 
  qf( -1, -0.13, 0.0, 0.0),
  p_fract( -1, -0.13, 0, 0),
  rndpix("./pixmaps/quat.xpm"),
  vrendpix("./pixmaps/vrend.xpm"),
  mouse_focus(0)
{  

  
  //TODO: do this somewhere else and some more variable */
  /* povray out init */
  /* this is hardcoded just for testing reasons */
  char* parfile = "povray/defines.tpl";
  char* pov_tplfile = "povray/ani.pov";
  char* ini_tplfile = "povray/quat.ini";

  /* read some template mapping file */
  string::size_type i, j;  
  ifstream from( parfile);
  if( !from) 
    cerr << "error cannot open " << parfile << endl;

  string tpl;
  map<string, string*> lookup;

  // parse in templates
  while( getline(from, tpl)) {
    i = tpl.find('=');
    if( i != string::npos)
      lookup[string(tpl.begin(), tpl.begin()+i)] = 
	new string( tpl.begin()+i+1, tpl.end());
  }


  /* initialize PovrayOut */
  po = manage( new PovrayOut( lookup, pov_tplfile, ini_tplfile));
  po->main_button.clicked.connect( bind<int>(slot(this,&QuatFractViewer::tool_handler),2));

  /* initialize GtkFractParams */
  gfp = manage( new GtkFractParams( qf.get_fparams()));
  gfp->main_button.clicked.connect( bind<int>(slot(this,&QuatFractViewer::tool_handler),4));
  gfp->update_request.connect( slot( this, &QuatFractViewer::update_params));
  gfp->redraw.connect(slot(this, &QuatFractViewer::drawgl));
  gfp->mouse_focus_request.connect(bind<int>(slot(this, &QuatFractViewer::mouse_manager),4));
  gfp->camera_stats_input.connect(slot(&camera, &SpaceEntity::get_stats));
  gfp->set_transient_for(*this);
  
  /* initialize render button */
  render_button.clicked.connect(bind<int>(slot(this,&QuatFractViewer::tool_handler),3));
  render_button.add(rndpix); 

  /* inititalize vrend button */
  vrend_button.clicked.connect(bind<int>(slot(this,&QuatFractViewer::tool_handler),6));
  vrend_button.add(vrendpix); 

  /* initialize PathMaster */
  pathmaster.main_button.clicked.connect( bind<int>(slot(this,&QuatFractViewer::tool_handler),1));
  pathmaster.camera_stats_input.connect(slot(&camera, &SpaceEntity::get_stats));
  pathmaster.redraw.connect(slot(this, &QuatFractViewer::drawgl));
  pathmaster.path_update.connect(slot(this, &QuatFractViewer::update_path));

  pathmaster.set_transient_for(*this);

  /* initialize Cutter */
  cutter.main_button.clicked.connect( bind<int>(slot(this,&QuatFractViewer::tool_handler),5));
  cutter.camera_stats_input.connect(slot(&camera, &SpaceEntity::get_stats));
  cutter.fparams_input.connect(slot(&qf, &QuatFractCreator::get_fparams));
  cutter.redraw.connect(slot(this, &QuatFractViewer::drawgl));
  cutter.mouse_focus_request.connect(bind<int>(slot(this, &QuatFractViewer::mouse_manager),5));

  cutter.set_transient_for(*this);

  /* initialize qfv_toolbar with QuatFractViewer tools... */
  using namespace Gtk::Toolbar_Helpers;
  qfv_toolbar.tools().push_back(Element(gfp->main_button,"edit fractal parameters"));
  qfv_toolbar.tools().push_back(Element(render_button,"render fractal!"));
  qfv_toolbar.tools().push_back(Element(pathmaster.main_button,"edit waypoints"));
  qfv_toolbar.tools().push_back(Element(po->main_button,"povray output"));
  qfv_toolbar.tools().push_back(Element(cutter.main_button,"make a 2D cut"));
  qfv_toolbar.tools().push_back(Element(vrend_button,"write vrend output"));
  

  /* ...pack it into a frame..*/
  qfv_toolbar_frame.set_border_width(10);
  qfv_toolbar_frame.set_shadow_type(GTK_SHADOW_ETCHED_IN);
  qfv_toolbar_frame.set_label(" QuatFract Tools");
  qfv_toolbar_frame.add(qfv_toolbar);
  
  /*... which is inserted into the main toolbar */
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Element(qfv_toolbar_frame));
  
  /* set up progress bar... */
  prog_bar.set_show_text( true);
  Viewer::lower_hbox.pack_start(prog_bar);

  /* ...and connect update signals */
  qf.render_progress.connect( slot( this, &QuatFractViewer::prog_bar_update));


  /* LightControl camera will be updated by this */
  //  light_control.refresh.connect( slot( this, &QuatFractViewer::light_control_update));


  /* choose rendering method */
  qf.set_method(CUBE);

  /* initialize cube algorithm cache */
  //TODO make static
  qf.cube_init();

  show_all();

  GenericViewer::initgl();  
}



void QuatFractViewer::mouse_manager(gboolean on, int id)
{
  switch(id){
    //4 is gfp
  case 4:
    if (on) mouse_focus=4;
    else mouse_focus=0;
    break;
  case 5:
    if (on) mouse_focus=5;
    else mouse_focus=0;
    break;
  }
}



void QuatFractViewer::tool_handler(int id) {
  switch(id) {
  case 1: 
    if(pathmaster.active==0) pathmaster.set_active(true);
    else pathmaster.set_active(false);
    break;
  case 2:
    if(po->active==0) po->set_active(true);
    else po->set_active(false);
    break;
  case 3:
    if (render_button.get_active()) render_fractal();
    break;
  case 4:
    if(gfp->active==0) gfp->set_active(true);
    else gfp->set_active(false);
    break;
  case 5:
    if(cutter.active==0) cutter.set_active(true);
    else cutter.set_active(false);
    drawgl();
    break;
  case 6:
    if (vrend_button.get_active()) qf.write_vrend("data.vrend");
    vrend_button.set_active( false);
    break;

  }
}


void QuatFractViewer::update_params() {
  qf.set_fparams( *gfp->get_params());

  glNewList(3, GL_COMPILE);
  draw_boundingbox();
  glEndList();

}


void QuatFractViewer::update_path() {
  po->update_path(pathmaster.get_spline());
  po->update_endtime( pathmaster.get_path().back()->time);
}

void QuatFractViewer::light_control_update() {
  light_control.camera = camera;
  light_control.gl_light_pos = gl_light_pos;
  light_control.updategl();
  light_control.drawgl();
}


void QuatFractViewer::prog_bar_update( float perc) {
  prog_bar.set_percentage( perc);
  while( Gtk::Main::events_pending())
    Gtk::Main::iteration();
}

gint QuatFractViewer::init_gl_context_imp(){

  /* call parent gl init */
  GenericViewer::init_gl_context_imp();

  GLfloat mat_specular[] = { 0., 0., 0., 1.0 };
  GLfloat mat_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);

  glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
  glEnable(GL_COLOR_MATERIAL);  
  
  glEnable(GL_LIGHTING);
  
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);    
  

  glNewList(2, GL_COMPILE);
  draw_coord();
  glEndList();

  glNewList(3, GL_COMPILE);
  draw_boundingbox();
  glEndList();


  return 0;
}


void QuatFractViewer::render_fractal() {

  int render_id, prev_id;
  pthread_t render_pt, prev_pt;

  /* render the display list(s) */
  cout << "rendering->" << endl;
  cout.flush();


  switch(qf.get_method()) {
  case CUBE:
    prog_bar.set_format_string( "cube polygonization  %p %%");

#if THREADING
    render_id = pthread_create(&render_pt, NULL, render_thread, (void*) this);
#else
    qf.cube_polygonize();
    prog_bar.set_format_string( "merging normals and drawing  %p %%");
    prog_bar_update( 0.99);
    QuatFractViewer::finish_render = false;
    if( display.make_current()) {
      glNewList(1, GL_COMPILE);
      qf.cube_merge_n_draw();
      glEndList();
    } else {  cerr << "make_current error" << endl;}
    QuatFractViewer::finish_render = true;
    prog_bar_update( 1.0);
#endif
    break;
  case TETRA:
    cout << "uncomment me" << endl;
    //qf.tetra_polygonize();
    break;
  case COMBI:
    cout << "implement me" << endl;
    //qf.combi_polygonize();
    break;
  case TRACK:
    cout << "implement me" << endl;
    //qf.track_polygonize();
    break;
  }
  drawgl();
  render_button.set_active(false);

  prog_bar.set_format_string( "rendering preview  %p %%");
 #if THREADING
   prev_id = pthread_create(&prev_pt, NULL, prev_thread, (void*) this);
 #else
   render_fractal_preview(10);
 #endif
   prog_bar_update( 1.0);
}


void QuatFractViewer::render_fractal_preview( int raster) {
  double min_diam;

  GLuint list_n = 10;

  p_fract = qf;
  FractParams p_fp = p_fract.get_fparams();

  /* get the minimal edge length of bounding box to
     have an acceptable raster */
  min_diam = p_fp.xMax - p_fp.xMin;
  min_diam = (p_fp.yMax - p_fp.yMin < min_diam) ? p_fp.yMax - p_fp.yMin : min_diam;
  min_diam = (p_fp.zMax - p_fp.zMin < min_diam) ? p_fp.zMax - p_fp.zMin : min_diam;
  
  p_fp.step = min_diam / raster;
  p_fract.set_fparams(p_fp);

  cout << "preview->" << endl;
  //  p_fract.init_conv_map();
  p_fract.cube_polygonize();

  if (light_control.display.make_current()) {
    light_control.finish_render = false;
    glNewList(list_n, GL_COMPILE);
      p_fract.cube_merge_n_draw();
    glEndList();
    light_control.finish_render = true;
  } else { cerr << "make_current error" << endl;}

  light_control.set_display_list( list_n);
  
}


gint QuatFractViewer::draw_gl_context_imp(){

  /* draw coordinate axes */
  glCallList(2);
  glCallList(3);

//   /* draw the Cube surrounding the iteration zone */
//   glColor3f(0, 1.0, 0);
//   glutWireCube (2*fparams.zMax);

  

  /* call the list to render */
  if( QuatFractViewer::finish_render)
    glCallList(1);

  pathmaster.draw();


  glDisable(GL_LIGHTING);

  if (cutter.active) cutter.draw();

  glPushMatrix();



  double* look_vec=GenericViewer::camera.lookat.get_vec();
  glColor3f(0.7,0.7,1);  
  glBegin(GL_LINES);
  glVertex3f(0,0,0);
  glVertex3f(look_vec[0],look_vec[1],look_vec[2]); 
  glEnd();

  glTranslatef(look_vec[0],look_vec[1],look_vec[2]);


  glutWireSphere(0.07,4,4);
  glEnable(GL_LIGHTING);
  glPopMatrix();
 
  //glutSolidTeapot(0.5);
  return 0;
}

gint QuatFractViewer::gl_motion_notify_handler_imp(GdkEventMotion* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render)
      GenericViewer::gl_motion_notify_handler_imp(event);
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_motion_imp(event);
    break;
  case 5:
    if( QuatFractViewer::finish_render);
    cutter.mouse_motion_imp(event);
    break;
  }
}


gint QuatFractViewer::gl_button_press_handler_imp(GdkEventButton* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render)
      GenericViewer::gl_button_press_handler_imp(event);
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_press_imp(event);
    break;
  case 5:
    if( QuatFractViewer::finish_render);
    cutter.mouse_press_imp(event);
    break;
  }
  
}

gint QuatFractViewer::gl_button_release_handler_imp(GdkEventButton* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render) {
      GenericViewer::gl_button_release_handler_imp(event);
      light_control_update();
    }
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_release_imp(event);
    break;
  case 5:
    if( QuatFractViewer::finish_render);
    cutter.mouse_release_imp(event);
    break;
  }
  
}


/* draw coordinate axes */
void QuatFractViewer::draw_coord() {

  glDisable(GL_LIGHTING);
  glBegin(GL_LINES);
    glColor3f(1, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 3, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 3, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 0, 3);
  glEnd();
  output(3, 0, 0, "x");
  output(0, 3, 0, "y");
  output(0, 0, 3, "z");
  glEnable(GL_LIGHTING);
}


/* draw bounding box in which fractal is rendered */
void QuatFractViewer::draw_boundingbox() {

  FractParams fp = qf.get_fparams();
  
  glDisable(GL_LIGHTING);
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glBegin(GL_QUAD_STRIP);
  glColor3f(0, 1, 0);
    glVertex3f( fp.xMin, fp.yMin, fp.zMin);
    glVertex3f( fp.xMax, fp.yMin, fp.zMin);
    glVertex3f( fp.xMin, fp.yMax, fp.zMin);
    glVertex3f( fp.xMax, fp.yMax, fp.zMin);
    glVertex3f( fp.xMin, fp.yMax, fp.zMax);
    glVertex3f( fp.xMax, fp.yMax, fp.zMax);
    glVertex3f( fp.xMin, fp.yMin, fp.zMax);
    glVertex3f( fp.xMax, fp.yMin, fp.zMax);
    glVertex3f( fp.xMin, fp.yMin, fp.zMin);
    glVertex3f( fp.xMax, fp.yMin, fp.zMin);

  glEnd();
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glEnable(GL_LIGHTING);

}



void* render_thread(void *pthis) {

  QuatFractViewer* qfv = (QuatFractViewer*) pthis;

  //  qfv->qf.init_conv_map();
  qfv->qf.cube_polygonize();

  QuatFractViewer::finish_render = false;
  
  if( qfv->display.make_current()) {
  glNewList(1, GL_COMPILE);
    qfv->qf.cube_merge_n_draw();
  glEndList();
  } else {  cerr << "make_current error" << endl;}
  QuatFractViewer::finish_render = true;
  qfv->drawgl();

  pthread_exit(0);
}



void* prev_thread(void *pthis) {
  QuatFractViewer* qfv = (QuatFractViewer*) pthis;
  qfv->render_fractal_preview(10);
  
  pthread_exit(0);
  
}



