/*
 *  QuatFractViewer.h
 *
 *  Fractal Viewer derived from Generic Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef FRACTAL_VIEWER
#define FRACTAL_VIEWER

#include <GL/glx.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <gtk--/button.h>
#include <gtk--/progressbar.h>

#include <math.h>

#include "GenericViewer.h"
#include "QuatFractCreator.h"
#include "widgets/Gtk_PovrayOut.h"
#include "widgets/Gtk_PathMaster.h"
#include "widgets/Gtk_FractalToolbox.h"
#include "widgets/Gtk_ViewerToolbox.h"
#include "widgets/Gtk_FractParams.h"
#include "widgets/Gtk_Cutter.h"
#include "OpenGLUtils.h"


void draw_coord(void);

void* render_thread(void*);
void* prev_thread(void*);



/** Quaternion fractal Viewer Class.
    This Viewer Class is specialiced for viewing quaternion
    fractals. It provides GTK+ widgets for easy parameter 
    modification and has child objects that implement 
    the mathematical methods and
    algorithms used to render the fractal shape.
*/
class QuatFractViewer: public GenericViewer {

  /** The QuatFractCreator */
  QuatFractCreator qf;

  /** Variable for blocking display update */
  static bool finish_render;

  /** The QuatFract Preview Creator */
  QuatFractCreator p_fract;
  
  /** Virtual methods, both drawing and mouse handling, are overriden */
  virtual gint init_gl_context_imp();

  /** Virtual methods, both drawing and mouse handling, are overriden */
  virtual gint draw_gl_context_imp();
  
  /** Virtual methods, both drawing and mouse handling, are overriden */
  virtual gint gl_motion_notify_handler_imp(GdkEventMotion* event); 

  /** Virtual methods, both drawing and mouse handling, are overriden */
  virtual gint gl_button_press_handler_imp(GdkEventButton* event); 

  /** Virtual methods, both drawing and mouse handling, are overriden */
  virtual gint gl_button_release_handler_imp(GdkEventButton* event); 
  
  /** Manages incoming mouse focus requests. Child widgets that get mouse focus
      must be derived from MouseControl. Then they have the proper methods to
      handle the mouse events, which will be send to them by the 
      QuatFractViewer. Thus, such child can have their own mouse control. */
  void mouse_manager(gboolean on,int id); 

  /** This variable monitors the current mouse focus. 0 means that the
      QuatFractViewer has control. */
  int mouse_focus; 
  Gtk::Frame qfv_toolbar_frame;
  /** For the GV tools */
  Gtk::Toolbar qfv_toolbar; 

  /** Button to start rendering */
  Gtk::ToggleButton render_button; 
  Gtk::Pixmap rndpix;

  /** Button for vrend output */
  Gtk::ToggleButton vrend_button; 
  Gtk::Pixmap vrendpix;
 
  /** Handler for tool signals */
  void tool_handler(int id);

  /** Update the QuatFractCreators parameters. Is connected to the 
      Gtk_FractParams window */
  void update_params(); 

  /** Updates the waypoint information of the Gtk_PovrayOutput. Is connected to 
      the Gtk_PathMaster window */
  void update_path();

  /** Draws a simple coordinate system. Will probably be moved elsewhere. */
  void draw_coord(); 

  /** Draws the renderlimit box. Will probably be moved elsewhere. */
  void draw_boundingbox();

  /** A new fractal is computed and rendered */
  void render_fractal();

  /** Update LightControl settings */
  void light_control_update();

  /** Update progress bar. Set it to perc percentage.*/
  void prog_bar_update( float perc);

  /** The PathMaster */
  PathMaster pathmaster;

  /** Our Povray Output Page */
  PovrayOut *po; 

  /** Window for changing fractparams */ 
  GtkFractParams *gfp;

  /** Window for making a 2d cut */
  Cutter cutter;

  /** Progress bar */
  Gtk::ProgressBar prog_bar;

  /* thread stuff */
  friend void* render_thread(void *);
  friend void* prev_thread(void *);


public:
  /** Constructor */
  QuatFractViewer(); 

  /** A fractal preview is computed and rendered. Used by Light Control Window */
  void render_fractal_preview( int raster);


};



#endif
