/*
 *  SpaceEntity.h
 *
 *  Entity in 3D Space
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef ITEM_H
#define ITEM_H

#include "Vector3.h" 
#include <gtkgl--/glarea.h>
#include <gtk--/base.h>

/** Space Status class.
    The data used by SpaceEntity packed together */
class SpaceStats
{
 public:
  Vector3 pos;
  Vector3 lookat;
  Vector3 up;
};

/** Space Entity class.
    A Class for 3D objects in the GL Space. Contains methods which ensure that
    the variables can be manipulated in a correct way (eg. the lookat-vector must
    not equal the position).
*/
class SpaceEntity
{
 public:
  /** Constructor */
  SpaceEntity(); 
  
  /** These methods make sure that everything is perpendicular and that up is 
      of unit length. */
  int set_pos(Vector3 p);
  
  /** see   int set_pos(Vector3 p).*/
  int set_lookat(Vector3 p);

  /** see   int set_pos(Vector3 p).*/   
  int set_up(Vector3 p);
  
  /** Returns the whole bunch; No reference! read only */
  SpaceStats get_stats();

  /** The stats are public so one can still manipulate them directly without the 
      computing done within the "set" methods to orthogonalize everyting. 
      Invalid oparations are not prevented, though! */
  Vector3 pos;

  /** See Vector3 pos.*/
  Vector3 lookat;

  /** See Vector3 pos.*/ 
  Vector3 up;

  /** These signals can be used to update external informations, eg within a 
      Gtk_CoordEntry. They are called within the "set" methods; keep in mind 
      that up might be changed without calling set_up due to ortogonalization 
      within set_pos and set_camera; here the signal can be used to ensure 
      actualization of external informations. */
  SigC::Signal1<int, Vector3>pos_update;

  /** See SigC::Signal1<int, Vector3>pos_update.*/
  SigC::Signal1<int, Vector3>lookat_update;

  /** See SigC::Signal1<int, Vector3>pos_update.*/
  SigC::Signal1<int, Vector3>up_update;
  

  //YES Thats funky; better ignore that
  void register_data (SigC::ScopeNode *)
    {
    };  
};


#endif
