/*
 *  Waypoint.h
 *
 *  waypoint entity
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef WAYPOINT_H
#define WAYPOINT_H

#include <vector>
#include "SpaceEntity.h"
#include "Vector3.h"


/** Desribes a wapoint on the camera flight path. Is derived from SpaceEntity,
    so it has position and orientation information, and additionaly some 
    waypoint specific information like time and a forth coordinate. */

class Waypoint: public SpaceEntity
{
 public:
  /** Constructor. */
  Waypoint();

  /** Time at which the Camera should arrive during its flight. */
  double time; 

  /** Our quaternion space is 4D, so the waypoint has an additional coordinate. */
  double x4; 
  
  /** Set the time. */
  int set_time(double p);

  /** Set the forth coordinate.*/
  int set_x4(double p); 

  /** Signal which is emitted when x4 has changed. */  
  SigC::Signal1<int, double>x4_update;

  /** Signal which is emitted when the time has changed. */  
  SigC::Signal1<int, double>time_update;

};



#endif
