/*
 *  permutations.cc
 *
 *  Permutation mathematics
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "permutations.h"

Perm::Perm() {
  size = 5;
  map_array = new unsigned int[size];
  for( int i = 0; i < size; i++) 
    map_array[i] = i;
}

Perm::Perm(size_t size) {
  this->size = size;
  map_array = new unsigned int[size];
  for( int i = 0; i < size; i++) 
    map_array[i] = i;
}


Perm::Perm(size_t ssize, perm_t *mmap) {
  size = ssize;
  map_array = new perm_t[size];
  for( int i = 0; i < size; i++) {
    map_array[i] = mmap[i];
  }
}

inline perm_t Perm::map(perm_t el) const{
  return map_array[el];
}

inline size_t Perm::get_size() const{
  return size;
}

Perm& Perm::operator*(const Perm &p) const{
  Perm *erg = new Perm(size);
  for( int i = 0; i < size; i++) {
    //    cout << i+1 << "->" << buf1.map(i)+1 << "->" << buf2.map(buf1.map(i))+1 << endl;
    erg->map_array[i] = p.map(map(i));
  }
  return *erg;
}

bool Perm::operator==(const Perm &p) const{

  for(int i = 0; i < size; i++) {
    if( map(i) != p.map(i))
      return 0;
  }
  return 1;
}    

bool Perm::operator!=(const Perm &p) const{
  return !( *this == p);
}

Perm& Perm::inv() const{
  Perm *erg = new Perm(size);

  for(int i = 0; i < size; i++) 
    erg->map_array[map(i)] = i;

  return *erg;
}  


void Perm::print() const{
  bool used_flag[size];
  for(int i = 0; i < size; i++) 
    used_flag[i] = 0;
  
  for(int i = 0; i < size; i++) {
    if( ! used_flag[i]) {
      used_flag[i] = 1;
      if(map(i) != i) { 
	printf("(%i", i+1);
	int k = i;
	while((k = map(k)) != i) {
	  printf("%i", k+1);
	  used_flag[k] = 1;
	}
	printf(")");
      }
    }
  }
}


int Perm::sign() const{
  
  int nsum, psum;
  
  nsum = psum = 1;

  for(int i = 4; i > -1; i--) 
    for(int j = i-1; j > -1; j--) {
      nsum *= i-j;
      psum *= map(i)-map(j);
    }
  return nsum / psum;
}


int Perm::ord() const{
  Perm id(size);
  Perm s = *this;
  int c = 1;

  while( s != id) { 
    s = s * *this;
    c++;
  }
  
  return c;
}


ostream& operator<<(ostream &s, const Perm &p) {
  
  bool used_flag[p.get_size()];
  for(int i = 0; i < p.get_size(); i++) 
    used_flag[i] = 0;
  
  for(int i = 0; i < p.get_size(); i++) {
    if( ! used_flag[i]) {
      used_flag[i] = 1;
      if(p.map(i) != i) { 
	s << "(" << i+1;
	int k = i;
	while((k = p.map(k)) != i) {
	  s << k+1;
	  used_flag[k] = 1;
	}
	s << ")";
      }
    }
  }
  
  return s;
}

