/*
 *  main.cc
 *
 *  main routine of QuatFract
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <iostream>
#include <vector>
#include <list>
#include <map>
#include <string>
#include <getopt.h>


#include "QuatMath.h"
#include "lib/permutations.h"
#include "QuatFractCreator.h"
#include "OpenGLUtils.h"
#include "Cube.h"
#include "QuatFractViewer.h"

#define QF_VERSION "1.0"


int main (int argc, char *argv[])
{


  /* argument parsing */
  struct option opts[] = {
    {"version", 0, 0, 'v'},
    {"help", 0, 0, 'h'}
  };
  int option_index;
  int c;

  while( (c = getopt_long( argc, argv, "vh", opts, &option_index)) != -1) {
    switch(c) {
    case 'v': 
      cout << "quatfract version " << QF_VERSION << endl;
      exit(0);
    case 'h':
      cout << "Usage: " << argv[0] << " [OPTION] ..." << endl << endl
	   << "\t-v,  --version,\t\tprint version number" << endl
	   << "\t-h,  --help,\t\tprint help" << endl << endl;
      exit(0);
    }
  }

  /* start the program */

  Cube::init_rot();

  Gtk::Main myapp(&argc, &argv);
  QuatFractViewer fv;

  myapp.run();
  return 0;
}

  
