/*
 *  Gtk_CameraControl.h
 *
 *  Camera Control widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef CAMERA_INFOBOX
#define CAMERA_INFOBOX

#include "Gtk_CoordEntry.h"
#include <gtk--/box.h>
#include <gtk--/window.h>
#include <gtk--/togglebutton.h>
#include <gtk--/button.h>
#include <gtk--/frame.h>
#include <gtk--/pixmap.h>
#include "Gtk_ToolWindow.h"

/** Camera controlling window. This window consists of severel CoordEntries that
    show the current camera stats, and two buttons which select the mouse modes
    available from the GenericViewer.
*/
class CameraControl : public ToolWindow
{
  Gtk::VBox vbox;
  Gtk::Frame button_frame;
  Gtk::VBox vbubox;
  Gtk::ToggleButton move_mode_button1;
  Gtk::ToggleButton rotate_mode_button2;
  /** for the main button */
  Gtk::Pixmap campix;

  /** Handler for the buttons.Emitts mode_set signal when selection changed. */
  void on_button_toggled( int id);

 public:

  /** Signal is emitted when the mode buttons were pressed. The integer 
      argument tells which mode was selected. */ 
  SigC::Signal1<void, int> mode_set;

  /** Entry boxes with information about the camera. */
  CoordEntry camera_Gtk_CoordEntry;

  /** Entry boxes with information about the camera. */
  CoordEntry lookat_Gtk_CoordEntry;

  /** Entry boxes with information about the camera. */
  CoordEntry up_Gtk_CoordEntry;

  /** Entry boxes with information about the camera. */
  CoordEntry light_Gtk_CoordEntry;

  /** Constructor.*/
  CameraControl(); 
};

#endif
