/*
 *  Gtk_CoordEntry.h
 *
 *  Coordinate Entry widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef COORD_ENTRY_H
#define COORD_ENTRY_H

#include <gtk--/label.h>
#include <gtk--/entry.h>
#include <gtk--/table.h>
#include "../Vector3.h"
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <list>
#include <string.h>

/** Entries to show a set of three coordinates. No non-digits can be entered. 
    Contains a signal which is emitted when one of the entries was activated 
    (Enter must be pressed!).
 */
class CoordEntry: public Gtk::Table
{
  Gtk::Label label;

  /** Single coordinate entry. */
  Gtk::Entry x_entry;

  /** Single coordinate entry. */
  Gtk::Entry y_entry;

  /** Single coordinate entry. */
  Gtk::Entry z_entry;
  
  /** Labels the corresponding entry.*/
  Gtk::Label x_label;

  /** Labels the corresponding entry.*/
  Gtk::Label y_label;

  /** Labels the corresponding entry.*/
  Gtk::Label z_label;

  /** Handler method. */
  void on_entry_activated();

  /** Here the coordinates are stored. */
  Vector3 coord;

  /** A variable to identify the CoordEntry. Due to the possibility to bind 
      integers to events when connecting it might be removed in the future. */ 
  int identifier;

public:

  /** Returns the identifier.*/
  int get_id(); 

  /** Signal which is emitted when enter was pressed within one of the entries. */
  SigC::Signal1<void, int>entry_activated; 

  /** Constructor. lab is a string which labels the whole widget and id is 
      assigned to the identifier variable. */
  CoordEntry(const char* lab, int id);

  /** Set the coordinates. */
  int set_coord(Vector3 c); 

  /** Returns the coordinates. */ 
  Vector3 get_coord(); 
};



#endif

