#include "Gtk_Cutter.h"


using SigC::bind;
using SigC::slot;




Cutter::Cutter():
  table(4,5,false),
  outtable(2,2,false),
  buframe("mouse control"),
  outframe("output window"),
  bu1_move("translate"),
  bu2_rotate("rotate"),
  bu3_resize("resize"),
  go_bu("go!"),
  up_a(GTK_ARROW_UP,GTK_SHADOW_OUT),
  down_a(GTK_ARROW_DOWN,GTK_SHADOW_OUT),
  cpix("./pixmaps/cutter.xpm"),
  pos_entry("position",1), 
  norm_entry("normal",2),
  up_entry("up",3),
  width_entry(" width ",4),
  height_entry(" height ",5),
  outw_entry("width",6),
  outh_entry("height",7)
{

  set_title("Fractal Cutter");

  //  out_win=NULL;
  //  out_display=NULL;
  out_win.add(out_display);
  out_win.set_policy(FALSE,FALSE,TRUE);

/* this are calls to ensure correct mouse handling. Must be called for some 
    reason before realize!*/
  out_display. set_events(GDK_EXPOSURE_MASK|
		      GDK_BUTTON_MOTION_MASK|
		      GDK_POINTER_MOTION_MASK|
		      GDK_POINTER_MOTION_HINT_MASK|
		      GDK_BUTTON_PRESS_MASK|
		      GDK_BUTTON_RELEASE_MASK);

  /* must be realized before any gl calls can be made */
  out_display.realize();

  width=1;
  height=1;

  outw=200;
  outh=200;

  out_mouse1_pressed=0;

  Vector3 pos_start(0,0,0);
  Vector3 lookat_start(1,0,0);
  Vector3 up_start(0,1,0);
  /*
  cut.set_pos(pos_start);
  cut.set_lookat(lookat_start);
  cut.set_up(up_start);
  */
  cut.pos=pos_start;
  cut.lookat=lookat_start;
  main_button.add(cpix);

  //  hbox.pack_start(pos_entry);
  //  hbox.pack_start(norm_entry);
  
  vbubox.pack_start(bu1_move);
  vbubox.pack_start(bu2_rotate);
  vbubox.pack_start(bu3_resize);

  buframe.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  buframe.set_border_width(6);  
  buframe.add(vbubox);  

  whhbox.pack_start(height_entry);
  whhbox.pack_start(width_entry);
  
  up_bu.add(up_a);
  down_bu.add(down_a);
  
  udhbox.pack_start(up_bu);
  udhbox.pack_start(down_bu);

  udbuframe.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  udbuframe.set_border_width(6);  
  udbuframe.add(udhbox);  

  up_tip.set_tip(up_bu,"apply ratio outwindowheight/outwindowsize to height/size");
  down_tip.set_tip(down_bu,"apply ratio height/size to outwindowheight/outwindowsize"); 

  outtable.attach(outh_entry,0,1,0,1);
  outtable.attach(outw_entry,1,2,0,1);
  outtable.attach(go_bu,0,2,1,2);
  outtable.set_row_spacing(0,5);
  
  outframe.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  outframe.set_border_width(6);  
  outframe.add(outtable);  



  table.attach(pos_entry,0,1,0,1);
  table.attach(norm_entry,1,2,0,1);
  table.attach(up_entry,0,1,1,2);
  table.attach(buframe,1,4,1,2);
  table.attach(whhbox,0,3,2,3);
  table.attach(udbuframe,0,3,3,4);
  table.attach(outframe,0,3,4,5);

  table.set_row_spacing(1,10);
  table.set_row_spacing(2,10);


  //  vbox.pack_start(hbox);
  //  vbox.pack_start(buframe);

  //  add(vbox);

  add(table);

  pos_entry.set_coord(cut.pos);
  norm_entry.set_coord(cut.lookat-cut.pos);
  up_entry.set_coord(cut.up);

  bu1_move.toggled.connect( bind<int>(slot(this, &Cutter::on_button_toggled), 1));

  bu2_rotate.toggled.connect( bind<int>(slot(this, &Cutter::on_button_toggled), 2));

  bu3_resize.toggled.connect( bind<int>(slot(this, &Cutter::on_button_toggled), 3));


  up_bu.clicked.connect( bind<int>(slot(this, &Cutter::on_button_toggled), 4));

  down_bu.clicked.connect( bind<int>(slot(this, &Cutter::on_button_toggled), 5));

  go_bu.clicked.connect(slot(this, &Cutter::do_cut));

  //the main button is not visible!
  out_win.main_button.clicked.connect(slot(this,&Cutter::toggle_out_win));

  out_display.expose_event.connect(slot(this,&Cutter::out_expose_handler));
  out_display.configure_event.connect(slot(this,&Cutter::out_configure_handler));
  out_display.motion_notify_event.connect(slot(this,&Cutter::out_motion_handler));
  out_display.button_press_event.connect(slot(this,&Cutter::out_press_handler));
  out_display.button_release_event.connect(slot(this,&Cutter::out_release_handler));


  pos_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));
  norm_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));
  up_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));

  width_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));
  height_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));
  outw_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));
  outh_entry.entry_activated.connect(slot(this,&Cutter::entry_activated));

  width_entry.set_value(width);
  height_entry.set_value(height);

  outw_entry.set_value(outw);
  outh_entry.set_value(outh);


}

void Cutter::on_button_toggled(int id)
{
  if(id<=3) //the mouse mode buttons
    {
      if (id==1&&bu1_move.get_active()) 
	{
	  bu2_rotate.set_active(false);
	  bu3_resize.set_active(false);
	  mode=1;
	  mouse_focus_request(true);
	}
      
      if (id==2&&bu2_rotate.get_active())
	{
	  bu1_move.set_active(false);
	  bu3_resize.set_active(false);
	  mode=2;
	  mouse_focus_request(true);
	}
      
      if (id==3&&bu3_resize.get_active())
	{
	  bu1_move.set_active(false);
	  bu2_rotate.set_active(false);
	  mode=3;
      mouse_focus_request(true);
	}
      
      
      if(!bu1_move.get_active()&&
	 !bu2_rotate.get_active()&&
	 !bu3_resize.get_active())
	{
	  mode=0;
	  mouse_focus_request(false);
	}
    }
  else //up/down button
    {

      if (id==4&&outh!=0&&outw!=0) //up
	{
	  if (outw<outh) width=(float)outw/outh*height;
	  else
	    { 
	      if (outw>outh) height=(float)outh/outw*width;
	      else // = ; take maximum
		{ 
		  if (height>width) width=height;
		  else height=width;
		}
	    }
	}
      
      else if (id==5&&height!=0&&width!=0) //down
	{
	  if (width<height) outw=(int)(width/height*outh);
	  else  
	    {
	      if (width>height) outh=(int)(height/width*outw);
	      else // = ; take maximum
		{
		  if (outh>outw) outw=outh;
		  else outh=outw;
		}
	    }
	}
      outw_entry.set_value(outw);
      outh_entry.set_value(outh);
      
      width_entry.set_value(width);
      height_entry.set_value(height);
      redraw();
    }
}


void Cutter::toggle_out_win()
{
  if (out_win.active==0) out_win.set_active(true);
  else out_win.set_active(false);
}


void Cutter::init_out()
{
}

void Cutter::update_out()
{
}

gint Cutter::draw_out()
{
  if (out_display.make_current())
    {

      glClearColor(0.0, 0.0, 0.0, 0.0); 
      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
      glCallList(1);

      //selection box
      if(out_mouse_start_x!=out_mouse_end_x && out_mouse_start_y!=out_mouse_end_y)
	{
      
	  int x0=(gint) out_mouse_start_x;
	  int x1=(gint) out_mouse_end_x;
	  //mouse coordinates is upside down
	  int y0=(gint) (outh-out_mouse_start_y);
	  int y1=(gint) (outh-out_mouse_end_y);
	  
	  
	  glColor3f(1,1,1);
	  glBegin(GL_LINE_STRIP);
	  glVertex3f(x0,y0,0);
	  glVertex3f(x0,y1,0);
	  glVertex3f(x1,y1,0);
	  glVertex3f(x1,y0,0);
	  glVertex3f(x0,y0,0);
	  glEnd();
	}
      out_display.swap_buffers();
    }
}


gint Cutter::out_expose_handler(GdkEventExpose* expose){
  /* we are CPU intensive so we should wait for the last event */
  /* well this comment is from somewhere else, actually i dont know what happends
     here exactly */
   if (expose->count>0) return 0;
   draw_out();
   return TRUE;
}

gint Cutter::out_configure_handler(GdkEventConfigure* configure){
   if (out_display.make_current())
     {
       update_out();

     }
 return TRUE;
}


gint Cutter::out_press_handler(GdkEventButton* event)
{
  if (event->button==1)
    {
      out_mouse_start_x=event->x;
      out_mouse_start_y=event->y;
      out_mouse1_pressed=1;
    }
}

gint Cutter::out_release_handler(GdkEventButton* event)
{
    out_mouse1_pressed=0;      
    out_mouse_end_x=event->x;
    out_mouse_end_y=event->y;
}

gint Cutter::out_motion_handler(GdkEventMotion* event)
{
  gint x,y;
  
  GdkModifierType state;

  /* the hint-thing is about filtering out some unnecessary events */
  
  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = (gint) event->x;
      y = (gint) event->y;
      state =(GdkModifierType) event->state;
    }
  
  if ((state & GDK_BUTTON1_MASK)&&out_mouse1_pressed 
      && out_mouse_start_x!=x && out_mouse_start_y!=y)

   {
     if (x<=0) x=0;
     else if (x>outw) x=outw;
     if (y<=0) y=0;
     else if (y>outh) y=outh;
     
     out_mouse_end_x=x;
     out_mouse_end_y=y;
     
     int x0=(gint) out_mouse_start_x;
     int x1=(gint) x;
     //mouse coordinates is upside down
     int y0=(gint) (outh-out_mouse_start_y);
     int y1=(gint) (outh-y);


     //computing new position & height & width
     //norm, start, side could be cached (side and norm added to SpaceEntity?)
     Vector3 norm=n;
     norm=norm/norm.abs();
     Vector3 side=cut.up^(norm);

     side=side/side.abs();
     

     Vector3 start=old_pos-side*(float)old_width/2-cut.up*(float)old_height/2;
     //     cerr<<side[0]<<" "<<side[1]<<" "<<side[2]<<endl;     

     height=(float)(y1-y0)/outh * old_height;
     width=(float)(x1-x0)/outw * old_width;

     cut.pos=start + cut.up*((float)y0/outh*old_height +height/2) + side*((float)x0/outw*old_width +width/2);
     
     cut.lookat=cut.pos+norm;

     if (height<0) height=-height;
     if (width<0) width=-width;

     update_out();
     draw_out();

     

     /* updating entries and redrawing the cutting plane */
     height_entry.set_value(height); 
     width_entry.set_value(width);
     pos_entry.set_coord(cut.pos);
     redraw();

   }
}


gint Cutter::iter_sqr(Quat &q) {

  for(int i = 0; i < fparams.maxiter; i++) {

    q.f(fparams.C);
    if( q.abs() > fparams.bailout)
      return i+1;
  }

  return 0;
}


gint Cutter::iter_cubic(Quat &q) {
  
  for(int i = 0; i < fparams.maxiter; i++) {
    q.g(fparams.C);

    if( q.abs() > fparams.bailout)
      return i+1;
  }

  return 0;
}







void Cutter::do_cut()
{
  fparams=fparams_input();

  out_win.set_usize(outw,outh);
  //  out_win.resize(outw,outh);

  out_win.main_button.set_active(true);
  out_win.set_title("2D Cut");

  //new cut so no selection box should be drawed
  out_mouse_start_x=0;
  out_mouse_start_y=0;
  out_mouse_end_x=0;
  out_mouse_end_y=0;

  old_width=width;
  old_height=height;
  old_pos=cut.pos;
  n=cut.lookat-cut.pos;
  n=n/n.abs();
  switch(fparams.iterFunc)
    {
    case SQR:
      konv = &Cutter::iter_sqr;
      break;
    case CUBIC:
      konv = &Cutter::iter_cubic;
      break;
    }

  
  if (out_display.make_current())
    {

      glViewport(0, 0, outw, outh);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluOrtho2D(0,outw,0,outh);

      glNewList(1, GL_COMPILE);


      Vector3 side=cut.up^n;
      side=side/side.abs();
      
      glBegin(GL_POINTS);
      Vector3 current;
      Vector3 start=cut.pos-side*width/2-cut.up*height/2;

      Vector3 upstep=cut.up*height/outh;
      Vector3 sidestep=side*width/outw;

      if (fparams.maxiter!=0){ //d be silly anyway
	
	for (int i=0;i<outw;i++)
	  {
	    current=start+sidestep*i;
      
	    for (int j=0; j<outh;j++)
	      {
		
		//warning this only works only as long our hyperplane is defined by x4=0      	      
		Quat q(current[0],current[1],current[2],0);
		
		gint iters=(this->*konv)(q);
		
		if (iters==0) //konvergent
		  glColor3f(0,0,atan(5*q.abs())/3.14*2);
		//	      glColor3f(0,0,log(q.abs()+1));
		
		else{ glColor3f((float)(fparams.maxiter-iters)/fparams.maxiter,0,0);}
		
		
		glVertex3f(i,j,0);
		current+=upstep;
	      }

	  }
      	glEnd();
	glEndList();
      }
    }

  
  
  out_win.show_all();
  draw_out();
}



gint Cutter::mouse_press_imp(GdkEventButton* event)
{
  camera_stats=camera_stats_input();
  
  if (event->button==1){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse1_pressed=1;
  }
  if (event->button==2){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse2_pressed=1;
  }
  
  if (event->button==3){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse3_pressed=1;
  }
  redraw();
}  

gint Cutter::mouse_release_imp(GdkEventButton* event)
{
   if (event->button==1)  
    {
     mouse1_pressed=0;
    }

 if (event->button==2)  
   {
     mouse2_pressed=0;
   }

 if (event->button==3)  
   {
     mouse3_pressed=0;
   }
  redraw();
}


gint Cutter::mouse_motion_imp(GdkEventMotion* event)
{

  gint x,y;
  
  GdkModifierType state;
  
  if (event->is_hint)
    gdk_window_get_pointer(event->window, &x, &y, &state);
  else
    {
      x = (gint) event->x;
      y = (gint) event->y;
      state =(GdkModifierType) event->state;
    }
  
  switch(mode)
    {
      
    case 1:
      
      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{
	  
	  Vector3 norm=cut.lookat-cut.pos;
	  Vector3 sidechange=(cut.up^norm)*width*(x-mouse_start_x)/500;
	  Vector3 upchange=cut.up*height*(mouse_start_y-y)/500;

	  cut.pos+=sidechange;
	  cut.pos+=upchange;
	  cut.lookat+=sidechange;
	  cut.lookat+=upchange;

	  mouse_start_x=x;
	  mouse_start_y=y;
	  
	  pos_entry.set_coord(cut.pos);
	  norm_entry.set_coord(cut.lookat-cut.pos);

	  redraw();  
	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{
	  
	  Vector3 normchange=(cut.lookat-cut.pos)*(mouse_start_y-y)*(width+height)/1000;

	  cut.pos+=normchange;
	  cut.lookat+=normchange;

	  mouse_start_x=x;
	  mouse_start_y=y;
	  
	  pos_entry.set_coord(cut.pos);
	  norm_entry.set_coord(cut.lookat-cut.pos);

	  redraw();  
	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{

	}

      break;

    case 2:

      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{
	  
	  Vector3 norm=cut.lookat-cut.pos;
	  Vector3 side=(cut.up^norm);
	  Vector3 up=cut.up;

	  norm=q_rotate(norm,(mouse_start_y-y)/200,side);

	  up=q_rotate(up,(mouse_start_y-y)/200,side);

	  norm=q_rotate(norm,(mouse_start_x-x)/200,up);

	  up=up/up.abs();//just to be sure
	  norm=norm/norm.abs(); //just to be sure
	  
	  cut.lookat=cut.pos+norm;
	  cut.up=up;


	  mouse_start_x=x;
	  mouse_start_y=y;
	  
	  up_entry.set_coord(up);
	  norm_entry.set_coord(norm);


	  redraw();  
	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{
	  Vector3 norm=cut.lookat-cut.pos;
	  Vector3 side=(cut.up^norm);
	  Vector3 up=cut.up;

	  up=q_rotate(up,(x-mouse_start_x)/200,norm);
	  up=up/up.abs();//just to be sure
	  mouse_start_x=x;
	  mouse_start_y=y;

	  up_entry.set_coord(up);
	  cut.up=up;

	  redraw();  
	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{

	}

      break;
      
    case 3:
      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{
	  
	  width*=exp((x-mouse_start_x)/500);
	  height*=exp((mouse_start_y-y)/500);
	  mouse_start_x=x;
	  mouse_start_y=y;
	  
	  width_entry.set_value(width);
	  height_entry.set_value(height);

	  
	  redraw();  
	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{

	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{

	}

      break;


    }
}



void Cutter::entry_activated(int id)
{
  Vector3 zero(0,0,0);
 //need to update up first as it might be changed later due to orthogonalization
  cut.set_up(up_entry.get_coord());

  cut.set_pos(pos_entry.get_coord());
  
  cut.set_lookat(cut.pos+norm_entry.get_coord()/(norm_entry.get_coord()).abs());


  norm_entry.set_coord(cut.lookat-cut.pos);

  up_entry.set_coord(cut.up);

  if (height_entry.get_value()!=0) height=height_entry.get_value();
  if (width_entry.get_value()!=0) width=width_entry.get_value();
  if (outh_entry.get_value()>1500) cerr<<"max height is 1000"<<endl;
  else if (outh_entry.get_value()!=0) outh=outh_entry.get_value();
  if (outw_entry.get_value()>1500) cerr<<"max width is 1000"<<endl;
  else if (outw_entry.get_value()!=0) outw=outw_entry.get_value();

  height_entry.set_value(height);
  width_entry.set_value(width);
  outh_entry.set_value(outh);
  outw_entry.set_value(outw);

  redraw();
}



void Cutter::draw()
{

  Vector3 norm=cut.lookat-cut.pos;
  Vector3 side=(cut.up^norm)*width;
  Vector3 up=cut.up*height;
  
  glPushMatrix();

  glTranslatef(cut.pos[0],cut.pos[1],cut.pos[2]);
    
  glColor3f(1,1,0);
  glBegin(GL_QUADS);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/3 + side[0]/2,
	     up[1]/3 + side[1]/2 ,
	     up[2]/3 + side[2]/2);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/(-2) + side[0]/2,
	     up[1]/(-2) + side[1]/2 ,
	     up[2]/(-2) + side[2]/2);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/(-2) - side[0]/2,
	     up[1]/(-2) - side[1]/2 ,
	     up[2]/(-2) - side[2]/2);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/3 - side[0]/2,
	     up[1]/3 - side[1]/2 ,
	     up[2]/3 - side[2]/2);

  glVertex3f(up[0]/2 + side[0]/3,
	     up[1]/2 + side[1]/3 ,
	     up[2]/2 + side[2]/3);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/(4) + side[0]/3,
	     up[1]/(4) + side[1]/3 ,
	     up[2]/(4) + side[2]/3);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/(4) - side[0]/2,
	     up[1]/(4) - side[1]/2 ,
	     up[2]/(4) - side[2]/2);
  glNormal3f(norm[0],norm[1],norm[2]);
  glVertex3f(up[0]/2 - side[0]/2,
	     up[1]/2 - side[1]/2 ,
	     up[2]/2 - side[2]/2);

  glEnd();

  glColor3f(1,0.5,0);
  glBegin(GL_LINES);
  glVertex3f(0,0,0);
  glVertex3f(norm[0],norm[1],norm[2]);
  glEnd();
  glBegin(GL_QUADS);
  glVertex3f(up[0]/2 + side[0]/3,
	     up[1]/2 + side[1]/3 ,
	     up[2]/2 + side[2]/3);
  glVertex3f(up[0]/2 + side[0]/2,
	     up[1]/2 + side[1]/2 ,
	     up[2]/2 + side[2]/2);
  glVertex3f(up[0]/3 + side[0]/2,
	     up[1]/3 + side[1]/2 ,
	     up[2]/3 + side[2]/2);
  glVertex3f(up[0]/3 + side[0]/3,
	     up[1]/3 + side[1]/3 ,
	     up[2]/3 + side[2]/3);

  glEnd();


  glPopMatrix();
}
