/*
 *  Gtk_Cutter.h
 *
 *  Makes a 2D cut through the fractal
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



#ifndef CUTTER
#define CUTTER

#include "Gtk_ToolWindow.h"
#include "../MouseControl.h"
#include "Gtk_CoordEntry.h"
#include "Gtk_FloatEntry.h"
#include "Gtk_PIntEntry.h"
#include "../Vector3.h"
#include "../SpaceEntity.h"
#include "../QuatMath.h"
#include "../QuatFractCreator.h"


#include <gtk--/frame.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/pixmap.h>
#include <gtk--/table.h>
#include <gtk--/arrow.h>
#include <gtk--/tooltips.h>

#include <gtkgl--/glarea.h>


#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>


class Cutter: public ToolWindow, public MouseControl 
{
  Gtk::Table table;
  Gtk::Table outtable;
  //  Gtk::VBox vbox;
  //  Gtk::HBox hbox;
  Gtk::VBox vbubox;
  Gtk::HBox whhbox;
  Gtk::HBox udhbox;
  Gtk::Frame buframe;
  Gtk::Frame udbuframe;
  Gtk::Frame outframe;
  Gtk::ToggleButton bu1_move;
  Gtk::ToggleButton bu2_rotate;
  Gtk::ToggleButton bu3_resize;
  Gtk::Button go_bu;
  Gtk::Arrow up_a;
  Gtk::Arrow down_a;
  Gtk::Button down_bu;
  Gtk::Button up_bu;
  Gtk::Tooltips up_tip;
  Gtk::Tooltips down_tip;



   // for the main button
  Gtk::Pixmap cpix;


  //ok only need the overriden destroy event for hide&show; main_button is not visible
  ToolWindow out_win;
  Gtk::GLArea out_display;


  CoordEntry pos_entry;
  CoordEntry norm_entry;
  CoordEntry up_entry;

  FloatEntry width_entry;
  FloatEntry height_entry;
  PIntEntry outw_entry;
  PIntEntry outh_entry;  


  /** Cut stats. lookat-pos is the normal. */ 
  SpaceEntity cut;

 /** Here the current camera stats are stored. Is needed for the mouse control . */
  SpaceStats camera_stats; 

  double width;
  double height;
  
  //needed for the selection within the out_window
  double old_width;
  double old_height;
  Vector3 old_pos;
  //normal
  Vector3 n;


  unsigned int outw;
  unsigned int outh;

  FractParams fparams;


  /** Tells which mode is currently active. Zero means that this widget has 
      currently no control. */
  int mode; 
 
  /** Handler for the mouse mode buttons. */
  void on_button_toggled( int id); 

  /** activate/deactivate. */
  void toggle_out_win(); 

  void do_cut();

  //returns number of iterations till bailout if convergent and else 0 (for coloring)
  gint iter_sqr(Quat &q);
  gint iter_cubic(Quat &q);

  gint (Cutter::* konv)(Quat& q);

 /* some variables for the mouse control of the out window */
  
  gdouble out_mouse_start_x, out_mouse_start_y;
   gdouble out_mouse_end_x, out_mouse_end_y;
  gboolean out_mouse1_pressed; 

  /** This method initializes the  specific GL environment. It is called 
      within the constructor [NOT needed right now]. 
      Right now there is no need to call this method otherwise. */
  void init_out(); 

  /** Updates all the gl stats.Not needed right now.*/ 
  void update_out();  

  /** Draws the cut. */
  gint draw_out();  

  /** This handling method is called when ever an expose event occurs, eg when
      the mouse enters the out-window (?) */
  gint out_expose_handler(GdkEventExpose* expose);

  /** Another handling method. It is called when the out-window is resized. */
  gint out_configure_handler(GdkEventConfigure* configure);


  /** Method which handels mouse motionin the out window. Draws a new selection. 
  */
  gint out_motion_handler(GdkEventMotion* event); 

  /** Method which handels mouse clicks in the out window.
   */ 
  gint out_press_handler(GdkEventButton* event);

  /** Method which handels mouse clicks in the out window.
   */
  gint out_release_handler(GdkEventButton* event);

 public:

  Cutter();
  

  /** This signal recieves the camera stats. */  
  SigC::Signal0<SpaceStats> camera_stats_input;  


  /** This signal recieves the fractal parameters. */  
  SigC::Signal0<FractParams> fparams_input;  


 /** When the stats of the cut have changed, the scene needs to be redrawn. */
  SigC::Signal0<int> redraw; 



 /** Here the mouse control is realized. */
  virtual gint mouse_motion_imp(GdkEventMotion* event); 
  
  /** Here the mouse control is realized. */
  virtual gint mouse_press_imp(GdkEventButton* event); 
  
  /** Here the mouse control is realized. */
  virtual gint mouse_release_imp(GdkEventButton* event); 


  /** One of the entries in was activated. Stores new information.*/
  void entry_activated(int id);


  /** Here the cut is drawn. Is called within the draw_gl_context_imp
      of the QuatFractViewer.*/
  void draw(); 

};



#endif 
