/*
 *  Gtk_FractParams.cc
 *
 *  Fractal parameters window
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_FractParams.h"

#include <stdio.h>

using SigC::bind;
using SigC::slot;


RenderOptionsPage::RenderOptionsPage(  const FractParams& fp) :
  hbox1(false,0),
  frame1("polygonization method"),
  mitem1("cube (works fine)"), 
  mitem2("tetra (uncomment me)"), 
  mitem3("combi (implement me)"), 
  mitem4("track (implement me)") ,
  move_mode_button1("move"),
  resize_mode_button2("resize")
{


  move_mode_button1.toggled.connect( bind<int>(slot(this, &RenderOptionsPage::on_button_toggled), 1));

  resize_mode_button2.toggled.connect( bind<int>(slot(this, &RenderOptionsPage::on_button_toggled), 2));

 


  pack_start(hbox1, FALSE, FALSE);

  list<char* > ll;

  ll.push_back( "lower bounds");
  ll.push_back( "x");
  ll.push_back( "y");
  ll.push_back( "z");

  min_bound = manage( new GroupEntry(ll));
  hbox1.pack_start(*min_bound);

  ll.clear();
  ll.push_back( "upper bounds");
  ll.push_back( "x");
  ll.push_back( "y");
  ll.push_back( "z");

  max_bound = manage( new GroupEntry(ll));
  hbox1.pack_start(*max_bound);

  mbox.pack_start(move_mode_button1);
  mbox.pack_start(resize_mode_button2);

  mouse_frame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  mouse_frame.set_border_width(6);  
  mouse_frame.set_label("mouse control");
  mouse_frame.add(mbox);  

  pack_start(mouse_frame);

  pack_start(sep1);

  ll.clear();
  ll.push_back( "raster");
  ll.push_back( "step");

  raster = manage( new GroupEntry(ll));
  pack_start(*raster);

  pack_start(frame1);
  frame1.add(om1);
  
  om1.set_menu(menu1);


  mitem1.activate.connect( bind<pm>(slot(this, &RenderOptionsPage::set_method), CUBE));
  mitem2.activate.connect( bind<pm>(slot(this, &RenderOptionsPage::set_method), TETRA));
  mitem3.activate.connect( bind<pm>(slot(this, &RenderOptionsPage::set_method), COMBI));
  mitem4.activate.connect( bind<pm>(slot(this, &RenderOptionsPage::set_method), TRACK));
  menu1.append( mitem1);
  menu1.append( mitem2);
  menu1.append( mitem3);
  menu1.append( mitem4);

  update_params( fp);

}  



void RenderOptionsPage::on_button_toggled(int id)
{

  if (id==1&&move_mode_button1.get_active()) 
    {
      resize_mode_button2.set_active(false);
      mouse_request(1);
    }

  if (id==2&&resize_mode_button2.get_active())
    {
      move_mode_button1.set_active(false);
      mouse_request(2);
    }

  if(!move_mode_button1.get_active()&&!resize_mode_button2.get_active())
    mouse_request(0);
}




void RenderOptionsPage::update_params(const FractParams& fp) {
  char str[20];
  snprintf( str, 20, "%lf", fp.xMin);
  min_bound->set_entry(0, str);
  snprintf( str, 20, "%lf", fp.yMin);
  min_bound->set_entry(1, str);
  snprintf( str, 20, "%lf", fp.zMin);
  min_bound->set_entry(2, str);
  snprintf( str, 20, "%lf", fp.xMax);
  max_bound->set_entry(0, str);
  snprintf( str, 20, "%lf", fp.yMax);
  max_bound->set_entry(1, str);
  snprintf( str, 20, "%lf", fp.zMax);
  max_bound->set_entry(2, str);
  snprintf( str, 20, "%lf", fp.step);
  raster->set_entry(0, str);

  switch( fp.polyMethod) {
  case CUBE:
    menu1.activate_item(mitem1, false);
    break;
  case TETRA:
    menu1.activate_item(mitem2, false);
    break;
  case COMBI:
    menu1.activate_item(mitem3, false);
    break;
  case TRACK:
    menu1.activate_item(mitem4, false);
    break;
  }

}	  


void RenderOptionsPage::get_params(FractParams* fp) {

  fp->xMin = atof( min_bound->get_entry(0).c_str());
  fp->yMin = atof( min_bound->get_entry(1).c_str());
  fp->zMin = atof( min_bound->get_entry(2).c_str());
  fp->xMax = atof( max_bound->get_entry(0).c_str());
  fp->yMax = atof( max_bound->get_entry(1).c_str());
  fp->zMax = atof( max_bound->get_entry(2).c_str());
  fp->step = atof( raster->get_entry(0).c_str());

  fp->polyMethod = method;

}	  


ParameterOptionsPage::ParameterOptionsPage(  const FractParams& fp) :
  func_frame("iteration function"),
  mitem1("X^2 + C"), 
  mitem2("X^3 + C"),
  x4_lab("x4"),
  x4(0.1, 3)
 {

  list<char* > ll;

  ll.push_back( "iteration");
  ll.push_back( "maxiter");
  ll.push_back( "bailout");

  iteration = manage( new GroupEntry(ll));
  pack_start(*iteration);

  pack_start(func_frame);
  func_frame.add(om1);
  
  om1.set_menu(menu1);

  mitem1.activate.connect( bind<iter_func>(slot(this, &ParameterOptionsPage::set_func), SQR));
  mitem2.activate.connect( bind<iter_func>(slot(this, &ParameterOptionsPage::set_func), CUBIC));
  menu1.append( mitem1);
  menu1.append( mitem2);

  ll.clear();
  ll.push_back( "C");
  ll.push_back( "q1"); 
  ll.push_back( "q2");
  ll.push_back( "q3");
  ll.push_back( "q4");

  constant = manage( new GroupEntry(ll));
  pack_start(*constant);

  x4_frame.add(x4_box);
  x4_box.pack_start(x4_lab);
  x4_box.pack_start(x4);
  x4.set_adjustment( new Gtk::Adjustment( 0, -1000, 1000, 0.1));

  pack_start(x4_frame);
  update_params( fp);

}  





void ParameterOptionsPage::update_params(const FractParams& fp) {
  char str[20];
  double *q;
  q = fp.C.get_vec();
  snprintf( str, 20, "%d", fp.maxiter);
  iteration->set_entry(0, str);
  snprintf( str, 20, "%f", fp.bailout);
  iteration->set_entry(1, str);

  x4.set_value( fp.x4);

  snprintf( str, 20, "%f", q[0]);
  constant->set_entry(0, str);
  snprintf( str, 20, "%f", q[1]);
  constant->set_entry(1, str);
  snprintf( str, 20, "%f", q[2]);
  constant->set_entry(2, str);
  snprintf( str, 20, "%f", q[3]);
  constant->set_entry(3, str);

  iterFunc = fp.iterFunc;
  
  switch( fp.iterFunc) {
  case SQR:
    menu1.activate_item(mitem1, false);
    break;
  case CUBIC:
    menu1.activate_item(mitem2, false);
    break;
  }


}	  


void ParameterOptionsPage::get_params(FractParams* fp) {
  Quat q(atof( constant->get_entry(0).c_str()),
	 atof( constant->get_entry(1).c_str()),
	 atof( constant->get_entry(2).c_str()),
	 atof( constant->get_entry(3).c_str()));
	 

  fp->maxiter = atoi( iteration->get_entry(0).c_str());
  fp->bailout = atof( iteration->get_entry(1).c_str());
  fp->C = q;
  fp->x4 = x4.get_value_as_float();

  fp->iterFunc = iterFunc;

}	  


 
GtkFractParams::GtkFractParams( const FractParams& fp) :
  vbox1(false,0),
  button1("update"),
  button2("cancel"),
  fppix("./pixmaps/param.xpm"),
  ro_page( fp), 
  po_page( fp)
{

  ro_page.mouse_request.connect(slot(this, &GtkFractParams::set_mouse_mode));

  this->fp = new FractParams( fp);

  set_title("Fractal Parameter Manipulation");

  ToolWindow::main_button.add(fppix);

  add(vbox1);

  vbox1.pack_start(nb);
  vbox1.pack_start(hbox1);

  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( ro_page, "Rendering"));
  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( po_page, "Parameters"));

  hbox1.pack_start(button1);
  hbox1.pack_start(button2);

  button1.clicked.connect( bind<int>(slot(this, &GtkFractParams::on_button_clicked), 1));
  button2.clicked.connect( bind<int>(slot(this, &GtkFractParams::on_button_clicked), 2));

}

void GtkFractParams::on_button_clicked( int id) {
  switch(id) {
  case 1:
    ro_page.get_params( fp);
    po_page.get_params( fp);
    update_request();
    redraw();
    break;
  case 2:
    ro_page.update_params( *fp);
    po_page.update_params( *fp);
    break;
  }
}

void GtkFractParams::set_mouse_mode(int m)
{
  mode=m;
  if (mode==0) mouse_focus_request(0);
  else mouse_focus_request(1);
}

gint GtkFractParams::mouse_motion_imp(GdkEventMotion* event)
{
  
  if (mode==1)
    {
      gint x,y;
      
      GdkModifierType state;
      
      if (event->is_hint)
	gdk_window_get_pointer(event->window, &x, &y, &state);
      else
	{
	  x = (gint) event->x;
	  y = (gint) event->y;
	  state =(GdkModifierType) event->state;
	}
      
      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{

	  /* along the coord axes
	   */
	  double xwidth=(fp->xMax-fp->xMin);	  
	  double xchange=(x-mouse_start_x)/500*xwidth;
	  fp->xMax+=xchange;
	  fp->xMin+=xchange;

	  double ywidth=(fp->yMax-fp->yMin);	  
	  double ychange=(mouse_start_y-y)/500*ywidth;
	  fp->yMax+=ychange;
	  fp->yMin+=ychange;
	  	  
	  /*along camera stats
	  double size=((fp->xMax-fp->xMin)
	    +(fp->yMax-fp->yMin)
	    +(fp->yMax-fp->yMin))/3;

	  Vector3 look=camera_stats.lookat-camera_stats.pos;
	  look=look/look.abs();
	  Vector3 up=camera_stats.up;
	  Vector3 side=up^look;
	  Vector3 change=side*(mouse_start_x-x)/500+up*(mouse_start_y-y)/500;
	  change*=size;

	  fp->xMax+=change[0];
	  fp->xMin+=change[0];

	  fp->yMax+=change[1];
	  fp->yMin+=change[1];

	  fp->zMax+=change[2];
	  fp->zMin+=change[2];
	  */

	  mouse_start_x=event->x;
	  mouse_start_y=event->y;
	  
	  ro_page.update_params( *fp);
	  update_request();
	  redraw();  

	  
	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{

	  //along the coord axes
	  double zwidth=(fp->zMax-fp->zMin);	  
	  double zchange=(y-mouse_start_y)/500*zwidth;
	  fp->zMax+=zchange;
	  fp->zMin+=zchange;
	  /*	  along camera stats
	  double size=((fp->xMax-fp->xMin)
	    +(fp->yMax-fp->yMin)
	    +(fp->yMax-fp->yMin))/3;

	  Vector3 look=camera_stats.lookat-camera_stats.pos;
	  look=look/look.abs();
	  Vector3 change=look*(mouse_start_y-y)/500;
	  change*=size;

	  fp->xMax+=change[0];
	  fp->xMin+=change[0];

	  fp->yMax+=change[1];
	  fp->yMin+=change[1];

	  fp->zMax+=change[2];
	  fp->zMin+=change[2];
	  */

	  mouse_start_x=event->x;
	  mouse_start_y=event->y;
	  
	  ro_page.update_params( *fp);
	  update_request();
	  redraw();  

	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{

	}
      
    }
  if(mode==2)
    {
      gint x,y;
      
      GdkModifierType state;
      
      if (event->is_hint)
	gdk_window_get_pointer (event->window, &x, &y, &state);
      else
	{
	  x = (gint) event->x;
	  y = (gint) event->y;
	  state =(GdkModifierType) event->state;
	}
      
      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{

	  {
	    double change=exp((x-mouse_start_x)/500);
	    double center=(fp->xMax+fp->xMin)/2;
	    double new_width=(fp->xMax-fp->xMin)*change;
	    fp->xMax=center+new_width/2;
	    fp->xMin=center-new_width/2;
	  }

	  {
	    double change=exp((mouse_start_y-y)/500);
	    double center=(fp->yMax+fp->yMin)/2;
	    double new_width=(fp->yMax-fp->yMin)*change;
	    fp->yMax=center+new_width/2;
	    fp->yMin=center-new_width/2;
	  }

	  mouse_start_x=event->x;
	  mouse_start_y=event->y;

	  ro_page.update_params( *fp);
	  update_request();
	  redraw();  

	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{

	  {
	    double change=exp((mouse_start_y-y)/100);
	    double center=(fp->zMax+fp->zMin)/2;
	    double new_width=(fp->zMax-fp->zMin)*change;
	    fp->zMax=center+new_width/2;
	    fp->zMin=center-new_width/2;
	  }
	  
	  mouse_start_x=event->x;
	  mouse_start_y=event->y;
	  
	  ro_page.update_params( *fp);
	  update_request();
	  redraw();  

	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{
	  
	}
      
    }
    

}


gint GtkFractParams::mouse_press_imp(GdkEventButton* event)
{
  camera_stats=camera_stats_input();
  
  if (event->button==1){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse1_pressed=1;
  }
  if (event->button==2){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse2_pressed=1;
  }
  
  if (event->button==3){
    mouse_start_x=event->x;
    mouse_start_y=event->y;
    mouse3_pressed=1;
    
    if (mode==1)
      {
	
    }
    if(mode==2)
      {
	
      }
  } 
}

gint GtkFractParams::mouse_release_imp(GdkEventButton* event)
{
   if (event->button==1)  
    {
     mouse1_pressed=0;
    }

 if (event->button==2)  
   {
     mouse2_pressed=0;
   }

 if (event->button==3)  
   {
     mouse3_pressed=0;
   }

 if (mode==1)
    {

    }
  if(mode==2)
    {

    }
}




FractParams* GtkFractParams::get_params() {
  return fp;
}


