/*
 *  Gtk_FractParams.h
 *
 *  Fractal parameters window
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef GTK_FRACTPARAMS_H
#define GTK_FRACTPARAMS_H

#include <iostream>

#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/entry.h>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <gtk--/table.h>
#include <gtk--/label.h>
#include <gtk--/separator.h>
#include <gtk--/text.h>
#include <gtk--/frame.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/notebook.h>
#include <gtk--/togglebutton.h>
#include <gtk--/checkbutton.h>
#include <gtk--/paned.h>
#include <gtk--/buttonbox.h>
#include <gtk--/optionmenu.h>
#include <gtk--/menu.h>
#include <gtk--/menuitem.h>
#include <gtk--/spinbutton.h>
#include <gtk--/label.h>
#include <gtk--/adjustment.h>


#include "Gtk_ToolWindow.h"
#include "../Vector3.h" 
#include "Gtk_GroupEntry.h"
#include "../QuatFractCreator.h"
#include "../MouseControl.h"
#include "../SpaceEntity.h"

class RenderOptionsPage : public Gtk::VBox {
  
  Gtk::HBox hbox1;
  
  GroupEntry *min_bound;
  GroupEntry *max_bound;
  GroupEntry *raster;

  Gtk::Frame frame1;
  Gtk::Frame mouse_frame;
  Gtk::HBox mbox;
  Gtk::OptionMenu om1;
  Gtk::Menu menu1;
  Gtk::MenuItem mitem1;
  Gtk::MenuItem mitem2;
  Gtk::MenuItem mitem3;
  Gtk::MenuItem mitem4;
  Gtk::HSeparator sep1;

  Gtk::ToggleButton move_mode_button1;
  Gtk::ToggleButton resize_mode_button2;


  pm method;
  
  /** Handler for the mouse mode buttons. */
  void on_button_toggled( int id); 

  void set_method(pm newmethod) { method = newmethod;}

 public:
  
  /** When one of the buttons is pressed, we ask for mouse focus with this signal.
      This signal will be connected to GtkFractParams, which itself emmits a 
      signal to ask the QuatFractViewer for mouse focus.*/ 
  SigC::Signal1<void, int> mouse_request; 
  void get_params(FractParams* fp);
  void update_params( const FractParams& fp);
  RenderOptionsPage( const FractParams& fp);
};



class ParameterOptionsPage : public Gtk::VBox { 
 
  GroupEntry *iteration;

  Gtk::Frame func_frame;
  Gtk::OptionMenu om1;
  Gtk::Menu menu1;
  Gtk::MenuItem mitem1;
  Gtk::MenuItem mitem2;
  Gtk::MenuItem mitem3;
  Gtk::MenuItem mitem4;

  Gtk::Frame x4_frame;
  Gtk::HBox x4_box;
  Gtk::Label x4_lab;
  Gtk::SpinButton x4;

  GroupEntry *constant;

  iter_func iterFunc;

  void set_func(iter_func newfunc) { iterFunc = newfunc;}

 public:
 
  void get_params(FractParams* fp);
  void update_params( const FractParams& fp);
  ParameterOptionsPage( const FractParams& fp);
 
};



/** Gtk Fractal Params Window.
    This window gives access to the fractal parameters stored
    in the FractParams child object of QuatFractCreator.
*/
class GtkFractParams : public ToolWindow, public MouseControl {

  /* Gtk stuff */
  Gtk::VBox vbox1;

  Gtk::Notebook nb;
  RenderOptionsPage ro_page;
  ParameterOptionsPage po_page;
  
  Gtk::HButtonBox hbox1;
  Gtk::Button button1;
  Gtk::Button button2;

  /** for the main button. */
  Gtk::Pixmap fppix; 
  
  /* other ..data */
  
  FractParams* fp;
  
  void on_button_clicked( int id);
  
  /** Sets the mouse mode to m. Emitts a signal to ask for mouse focus if m!= 0,
      if m=0 the mouse focus is released.*/
  void set_mouse_mode(int m); 
  
  /** Tells which mode is currently active. Zero means that this widget has 
      currently no control. */
  int mode; 
  
  /** Here the current camera stats are stored. Might be used for a better control
      of the boundary box. */
  SpaceStats camera_stats; 
  
 public:
  
  GtkFractParams( const FractParams& fp);
  
  SigC::Signal0<void> update_request;
  
  /** Signal which will be connected to QuatFractViewers drawgl. The scene needs
      to be redrawn when the boundary box has changed. */
  SigC::Signal0<int> redraw; 
  
  /** This signal recieves the camera stats. */  
  SigC::Signal0<SpaceStats> camera_stats_input;  
  
  /** Here the mouse control over the boundary box is realized. */
  virtual gint mouse_motion_imp(GdkEventMotion* event); 
  
  /** Here the mouse control over the boundary box is realized. */
  virtual gint mouse_press_imp(GdkEventButton* event); 
  
  /** Here the mouse control over the boundary box is realized. */
  virtual gint mouse_release_imp(GdkEventButton* event); 
  
  FractParams* get_params();
  
};



#endif
