/*
 *  Gtk_FractalToolbox.h
 *
 *  Fractal Toolbox frame in QuatFractViewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef FRACTALTOOLS_H
#define FRACTALTOOLS_H

#include <gtk--/window.h>
#include <gtk--/box.h>
#include <gtk--/toolbar.h>
#include <gtk--/pixmap.h>
#include <gtk--/imageloader.h>
#include <gtk--/frame.h>

/** The Toolbox consists of a frame with several buttons in it, which will be 
    linked to QuatFract specific widgets. It is packed into the toolbox of 
    GeneriCViewer by the QuatFractViewer.
*/
class FractalToolbox: public Gtk::Frame
{
  
  /** In this array is stored which button is currently toggled. Toggling doesn't
      work correctly yet. */
  bool toggled[20]; 

  /** Handels a button change. Emits the tool_event. */
  void tool_handler(int id);

  /* The buttons are stored in a toolbar. */
  Gtk::Toolbar toolbar; 

  /* Pixmaps for the buttons must be stored here */
  Gtk::Pixmap waypix; 
  Gtk::Pixmap povpix;
  Gtk::Pixmap quatpix;
  Gtk::Pixmap parampix;

 public:

  /** Constructor. */
  FractalToolbox();

  /** A button has ben toggled. */
  SigC::Signal1<void,int>tool_event; 

  /** Returns if button is currently toggled. */
  bool get_toggle( int i) { return toggled[i];} 

};




#endif
