/*
 *  Gtk_GroupEntry.cc
 *
 *  Group of entry widgets
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_GroupEntry.h"

using SigC::bind;
using SigC::slot;


GroupEntry::GroupEntry( list<char* >& strlist, 
			int entry_width = 80, 
			int height = 22,
			int length = 6) :
  hbox(false, 0)
{
  list<char* >::iterator i = strlist.begin();
  int j = 0;
  
  set_border_width(10);
  set_shadow_type(GTK_SHADOW_ETCHED_IN);
  set_label(*i);
  i++;
  
  hbox.set_spacing(4);

  Gtk::VBox* vbox1 = manage( new Gtk::VBox(false, 0));
  Gtk::VBox* vbox2 = manage( new Gtk::VBox(false, 0));
 
  vbox1->set_spacing(1);
  vbox1->set_border_width(2);
  vbox2->set_spacing(1);
  vbox2->set_border_width(2);
 
  for( ; i != strlist.end(); i++, j++) {

    Gtk::Label* label = manage( new Gtk::Label( *i));
    Gtk::Entry* entry = manage( new Gtk::Entry);

    entry->set_usize(entry_width, height);
    entry->set_max_length(length);
    entry->activate.connect(bind<int >(slot(this, &GroupEntry::on_entry_activated), j));

    label->set_usize(-1, height);
    label->set_alignment(5, 0);



    ll.push_back( label);
    el.push_back( entry);
    vbox2->pack_start( *entry, false, false, 0);
    vbox1->pack_start( *label, false, false, 0);

  }

  hbox.pack_start( *vbox1, true, false, 0);
  hbox.pack_start( *vbox2, true, false, 0);

  add(hbox);
}
    

void GroupEntry::on_entry_activated(int j) {
  entry_activated(j);
}
