/*
 *  Gtk_PIntEntry.h
 *
 *  Positive Integer Entry widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef PINT_ENTRY_H
#define PINT_ENTRY_H

#include <gtk--/label.h>
#include <gtk--/entry.h>
#include <gtk--/box.h>
#include "../Vector3.h"
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <list>
#include <string.h>

/** Just a single entry that accepts positive integers only. 
 */

class PIntEntry: public Gtk::HBox
{
  Gtk::Entry entry;
  Gtk::Label label;

  /** Handle. */
  void on_entry_activated();

  /** Data. */
  unsigned int x; 

/** A variable to identify the PIntEntry. Due to the possibility to bind 
      integers to events when connecting it might be removed in the future. */ 
  int identifier;


public:

  /** Returns the identifier.*/
  int get_id();

  /** Signal which is emitted when enter was pressed within the entry. */
  SigC::Signal1<void, int>entry_activated;

  /** Constructor. lab is a string which labels the whole widget and id is 
    assigned to the identifier variable. */
  PIntEntry(const char* lab, int id);

  /** Set the value. */
  int set_value(unsigned int c);

  /** Returns the value. */
  unsigned int get_value();
};



#endif

