/*
 *  Gtk_PathMaster.cc
 *
 *  Master of waypoint path
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_PathMaster.h"
#include "../SplineMath.h"


using SigC::bind;
using SigC::slot;




PathMaster::PathMaster(): 
  table(7,3,false),
  get_cam_b("get stats from Camera"), 
  insert_b("insert waypoint"),
  replace_b("apply to\n current waypoint"),
  erase_b("erase waypoint"),
  spline_b("calculate splines"),
  pos("Position",1), 
  lookat("LookAt",2), 
  up("Up",3),
  x4_lab("x4"),
  x4(0.1, 3),
  time_lab("time"),
  time(1,2),
  pmpix("./pixmaps/smAnt.xpm"),
  a_up(GTK_ARROW_UP, GTK_SHADOW_OUT),
  a_down(GTK_ARROW_DOWN, GTK_SHADOW_OUT)
{
  next_wp_b.add( a_up);
  prev_wp_b.add( a_down);
	    

  vbbox.pack_start(insert_b);
  vbbox.pack_start(replace_b);
  vbbox.pack_start(erase_b);
  frame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  frame.set_border_width(6);  
  frame.add(vbbox);  
  wp_entry.set_usize(100,-1);
  wp_entry.set_editable(false);
  vwpbox.pack_start(next_wp_b);
  vwpbox.pack_start(wp_entry);
  vwpbox.pack_start(prev_wp_b);
  vwpbox.pack_start(spline_b);
  wpframe.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  wpframe.set_border_width(6);  
  wpframe.add(vwpbox);  

  table.attach(get_cam_b,0,2,0,1);
  table.attach(x4_lab,0,1,1,2);
  table.attach(x4,1,2,1,2);
  table.attach(time_lab,0,1,2,3);
  table.attach(time,1,2,2,3);
  table.attach(pos,2,3,0,3);
  table.attach(lookat,3,4,0,3);
  table.attach(up,4,5,0,3);
  table.attach(frame,5,6,0,3);
  table.attach(wpframe,6,7,0,3);

  add(table);

  insert_b.clicked.connect(slot(this,&PathMaster::add_waypoint));
  replace_b.clicked.connect(slot(this,&PathMaster::replace_waypoint));
  erase_b.clicked.connect(slot(this,&PathMaster::delete_waypoint));
  spline_b.clicked.connect(slot(this,&PathMaster::calculate_splines));
  next_wp_b.clicked.connect(slot(this,&PathMaster::next_waypoint));  
  prev_wp_b.clicked.connect(slot(this,&PathMaster::prev_waypoint));
  get_cam_b.clicked.connect(slot(this,&PathMaster::get_camera_pressed));
  pos.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  lookat.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  up.entry_activated.connect(slot(this,&PathMaster::entry_activated));


  x4.set_adjustment( new Gtk::Adjustment( 0, -1000, 1000, 0.1));
  time.set_adjustment( new Gtk::Adjustment( 0, 0, 1000, 0.1));
      
  /* connecting changes of our new_wp to the text entries. */
      
  new_wp.pos_update.connect(slot(this->pos,&CoordEntry::set_coord));
  new_wp.lookat_update.connect(slot(this->lookat,&CoordEntry::set_coord));
  new_wp.up_update.connect(slot(this->up,&CoordEntry::set_coord));
  
  set_title("Waypoint Editor");
      
  /*
      / setting up the entries with the stats of the "current" waypoint if path is
	 not empty /
      if(current!=-1){
	new_wp.set_pos(path[current]->pos); 
	new_wp.set_lookat(path[current]->lookat); 
	new_wp.set_up(path[current]->up);  
	new_wp.set_time(path[current]->time);
	new_wp.set_x4(path[current]->x4);
	
	editor->update_coord_entries(new_wp); 
	char j[11];
	if (current==-1) sprintf(j,"",current);
	else sprintf(j,"WayPoint%i",current);
	editor->wp_entry.set_text(j);
   }
  */  
  
  
  active=0;
  
  /* current=-1 means that the path empty */
  current=-1;
  
  main_button.add(pmpix);
}


void PathMaster::update_coord_entries(const Waypoint& wp)
{
 pos.set_coord(wp.pos);
 lookat.set_coord(wp.lookat);
 up.set_coord(wp.up); 
 time.set_value(wp.time);
 x4.set_value(wp.x4);
}

 
void PathMaster::add_waypoint()
{
  
  Waypoint* wp= new Waypoint;

  new_wp.set_x4( x4.get_value_as_float());
  new_wp.set_time( time.get_value_as_float());

  wp->pos=new_wp.pos;
  wp->lookat=new_wp.lookat;
  wp->up=new_wp.up;
  //SET THIS TO CURRENT X4 OF THE FRVIEWER
  wp->x4=new_wp.x4;

  current++;

  wp->time =new_wp.time ;   

  /* adding new wp */  
  path.insert(path.begin()+current,wp);

  /* setting the entries to current wps stats */
  char j[11];
  sprintf(j,"WayPoint%i",current);
  wp_entry.set_text(j);
  update_coord_entries(*path[current]); 

  /* emit neccesary signals */
  redraw(); 
}

void PathMaster::delete_waypoint()
{

  /* kind of ugly but seems to work */

  if (path.empty()) return ;
  current--;
  if(path.size()>1||current==0){
    path.erase(path.begin() +current+1);
    if (path.size()>0&&current==-1) current++;
  }
  else{
    path.erase(path.begin() +current+1);
  }
  char j[11];
  if (current==-1) sprintf(j,"",current);
  else sprintf(j,"WayPoint%i",current);

  wp_entry.set_text(j);

  if (current!=-1)
    {
      new_wp=*path[current];
      update_coord_entries(*path[current]); 
    }


  /* emit neccesary signals */
     
   redraw();

}


void PathMaster::replace_waypoint()
{
  if (path.empty()||current==-1) return;

  new_wp.set_x4( x4.get_value_as_float());
  new_wp.set_time( time.get_value_as_float());
  *path[current]=new_wp;
   redraw();
}


void PathMaster::get_camera_pressed()
{
  SpaceStats test;
  test=camera_stats_input();
  new_wp.set_pos(test.pos); 
  new_wp.set_lookat(test.lookat); 
  new_wp.set_up(test.up); 
  if (current!=-1) 
    {
     new_wp.set_time(path[current]->time+1);
     new_wp.set_x4(path[current]->x4);
    }
  update_coord_entries(new_wp); 
}


void PathMaster::entry_activated(int id)
{
  new_wp.set_up(up.get_coord());

  new_wp.set_pos(pos.get_coord());
   
  new_wp.set_lookat(lookat.get_coord());


}





void PathMaster::next_waypoint()
{
  if (current==path.size()-1) return;
    current++;
  char j[11];
   sprintf(j,"WayPoint%i",current);
   wp_entry.set_text(j);
   update_coord_entries(*path[current]); 

   /* emit redraw signal to display the new selection */
   redraw();
}

void PathMaster::prev_waypoint()
{
  if(current<=0) return;
  current--;
  char j[11];
   sprintf(j,"WayPoint%i",current);
   wp_entry.set_text(j);
   update_coord_entries(*path[current]); 

   /* emit redraw signal to display the new selection */
   redraw();
}

void PathMaster::calculate_splines()
{
  spline_math.update_path( path);
  spline_math.get_splines();
  spline_math.print_splines();
  redraw();
  path_update();
}


//own objekt for the gl stuff? display lists?
void PathMaster::draw()
{
  for(int i=0;i<path.size();i++)
    {
      glEnable( GL_NORMALIZE );
      glPushMatrix();
      glTranslatef(path[i]->pos[0],path[i]->pos[1],path[i]->pos[2]);
      glColor3f(1,0,0);
      Vector3 look0=(path[i]->lookat-path[i]->pos);
      Vector3 look=look0/look0.abs();
      Vector3 side=  path[i]->up ^ look;

      /* drawing the ugly waypoint */

      glBegin(GL_TRIANGLES);
      //got to simplify this
      Vector3 normal=(path[i]->up/10+(path[i]->up/10-side/8))^look;
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);


      normal=side ^ (path[i]->up/10+(path[i]->up/10-look/8));
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);

      normal=look^(path[i]->up/10+(path[i]->up/10+side/8));
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);


      normal=(path[i]->up/10+(path[i]->up/10+look/8))^side;
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);

      glEnd();

      glBegin(GL_QUADS);
      normal=path[i]->up/(-10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);

      glEnd();

      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
      glVertex3f(0,0,0);
      glVertex3f(look0[0],look0[1],look0[2]);
      glEnd();	


      /* drawing the ugly selection box */
      if(i==current&&active==1)
	{
	  glColor3f(0,1,0);
	  glutWireCube(0.2);
      }
      glEnable(GL_LIGHTING);
      glPopMatrix();
      glDisable( GL_NORMALIZE );
    }

  spline_math.render_splines();
}





//don't look
void PathMaster::register_data (SigC::ScopeNode *)
{
 
}

