/*
 *  Gtk_PathMaster.h
 *
 *  Master of waypoint path
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef PATHMASTER_H
#define PATHMASTER_H


#include "../Waypoint.h"
#include "../SplineMath.h"

#include "Gtk_FloatEntry.h"
#include "Gtk_CoordEntry.h"
#include "Gtk_ToolWindow.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <gtk--/window.h>
#include <gtk--/togglebutton.h>
#include <gtk--/buttonbox.h>
#include <gtk--/entry.h>
#include <gtk--/table.h>
#include <gtk--/frame.h>
#include <gtk--/button.h>
#include <gtk--/box.h>
#include <gtk--/pixmap.h>
#include <gtk--/arrow.h>
#include <gtk--/spinbutton.h>
#include <gtk--/label.h>
#include <gtk--/adjustment.h>

/** This class manages the camera flight path.  
    The PathMaster contains the path of waypoints.*/

class PathMaster: public ToolWindow 
{
  Gtk::Table table;
  Gtk::Frame frame;
  Gtk::Frame wpframe;
  Gtk::Button get_cam_b;
  Gtk::Button insert_b;
  Gtk::Button replace_b;
  Gtk::Button erase_b;
  Gtk::Button next_wp_b;
  Gtk::Button prev_wp_b;
  Gtk::Button spline_b;
  Gtk::VBox vbbox;
  Gtk::VBox vwpbox;
  Gtk::Entry wp_entry;
  CoordEntry pos;
  CoordEntry lookat;
  CoordEntry up;
  Gtk::Label x4_lab;
  Gtk::SpinButton x4;
  Gtk::Label time_lab;
  Gtk::SpinButton time;

  Gtk::Arrow a_up;
  Gtk::Arrow a_down;
  
  /** Set the entries with the stats of wp.*/
  void update_coord_entries(const Waypoint& wp);
  
  /** Here all the waypoints are stored. */
  vector<Waypoint*> path; 
  
  /** A new Waypoint (defined through the entries of the Editor window, stored in
      new_wp) is inserted into the path after the one defind by "current". This 
      method is connected to the corresponding button of the WapoinEditor window. */
  void add_waypoint(); 

  /** The "current" Waypoint is deleted. This method is connected to the 
      corresponding button of the WapoinEditor window. */
  void delete_waypoint(); 

  /** The "current" Waypoint is replaced by a new one with the stats defined 
      through the entries in the Editor window (stored in new_wp). This method is
      connected to the corresponding button of the WapoinEditor window. */
  void replace_waypoint();

  /** The next Waypoint within the path is selected and set current.*/
  void next_waypoint();

  /** The previous Waypoint within the path is selected and set current.*/
  void prev_waypoint();

  /** Calculate splines through waypoints.*/
  void calculate_splines();
  
  /** Defines the current Waypoint choosed within the editor. */
  int  current; 
  
  /** Here all the stats which are currently in the entries of the editor are
      stored. */
  Waypoint new_wp; 

  /** Spline mathematics. Used to interpolate spline and render it.*/
  SplineMath spline_math;

  // for the main button
  Gtk::Pixmap pmpix;

 public:
  /** Constructor.*/
  PathMaster();  

  /** This signal is supposed to recieve the current stats of the camera, which
      are needed for the "get stats from camera" button. */
  SigC::Signal0<SpaceStats> camera_stats_input; 

  /** When the stats of a Waypoint have changed, the scene needs to be redrawn. */
  SigC::Signal0<int> redraw; 

 
  //   SigC::Signal0<void> toggle_my_tool;
  
  /** This signal passes waypoint changes to the Gtk_PovrayOut (connection is 
      done by the QuatFractViewer).*/
  SigC::Signal0<void> path_update;

  /** The stats recieved from path_update signal are stored within new_wp. */
  void get_camera_pressed();

  /** One of the entries in the editor was activated. Store valid information
      within new_wp.*/
  void entry_activated(int id);

  //Ignore this
  void register_data (SigC::ScopeNode *);
  

  /** Here the wapoints are drawn. Is called within the draw_gl_context_imp
      of the QuatFractViewer.*/
  void draw(); 
  
  /** Returns the path. */
  vector< Waypoint*> get_path() { return path;}

  /** Returns the waypoint spline. */
  vector< SplinePart*>* get_spline() { 
    return spline_math.get_spline_data();
  }


};







#endif
