/*
 *  Gtk_PovrayOut.cc
 *
 *  Povray Output window
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_PovrayOut.h"


#define DEBUG 1
#define FLT_SIZE 7 


using SigC::bind;
using SigC::slot;


/* fills the lookup entries into tplfile using the marking !*****! */
string* fill_tpl( map<string, string*>& lookup, 
		  string tplstr, 
		  char marking = '!'){
  string *tpl = new string(tplstr);
  string::size_type i, j;

  while( (i = tpl->find('!')) != string::npos) {
    j = string(tpl->begin()+i+1, tpl->end()).find( '!');
    if( j != string::npos) {
      if( string* st = lookup[string(*tpl, i+1, j)])
	tpl->replace(i, j+2, *st);
      else {
	cerr << "error: template " << string(*tpl, i+1, j) 
	     << " not found in lookup map" << endl;
	return new string("NOT FOUND");
      }
    }
  }

  return tpl;
}


/* returns the povray ini section that uses clocked phases to 
   drive along our waypoint linestrip and set the parameters 
   according to clock and waypoints */
string* get_povray_path( vector< SplinePart*>* spl) {

  string tpl = "#range (!R1!, !R2!)\n";
  tpl += "\tlocation !A! + !B! * (clock - !R1!) + "; 
  tpl += "!C!/2 * pow(clock - !R1!, 2) + ";
  tpl += "!D!/6 * pow(clock - !R1!, 3)\n"; 
  tpl += "\tsky !A2! + !B2! * (clock - !R1!) + "; 
  tpl += "!C2!/2 * pow(clock - !R1!, 2) + ";
  tpl += "!D2!/6 * pow(clock - !R1!, 3)\n"; 
  tpl += "\tlook_at !A1! + !B1! * (clock - !R1!) + "; 
  tpl += "!C1!/2 * pow(clock - !R1!, 2) + ";
  tpl += "!D1!/6 * pow(clock - !R1!, 3)\n"; 
  tpl +=  "#break\n";
  

    
  char c[FLT_SIZE];
  map<string, string*> path_map;
  string tmp;
  string* pp = new string;

  *pp = "#switch(clock)\n";

  for( int i = 0; i < spl->size()-1; i++) {
    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->t);    
    tmp += c;
    path_map["R1"] = new string(tmp);
    if(DEBUG) cout << "R1=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i+1]->t);    
    tmp += c;
    path_map["R2"] = new string(tmp);
    if(DEBUG) cout << "R2=" << tmp << endl;
    
    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[2].a);
    tmp += c;
    tmp += ">";
    path_map["A"] = new string(tmp);
    if(DEBUG) cout << "A=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[2].b);
    tmp += c;
    tmp += ">";
    path_map["B"] = new string(tmp);
    if(DEBUG) cout << "B=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[2].c);
    tmp += c;
    tmp += ">";
    path_map["C"] = new string(tmp);
    if(DEBUG) cout << "C=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[2].d);
    tmp += c;
    tmp += ">";
    path_map["D"] = new string(tmp);
    if(DEBUG) cout << "D=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[3].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[4].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[5].a);
    tmp += c;
    tmp += ">";
    path_map["A1"] = new string(tmp);
    if(DEBUG) cout << "A1=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[3].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[4].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[5].b);
    tmp += c;
    tmp += ">";
    path_map["B1"] = new string(tmp);
    if(DEBUG) cout << "B1=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[3].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[4].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[5].c);
    tmp += c;
    tmp += ">";
    path_map["C1"] = new string(tmp);
    if(DEBUG) cout << "C1=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[3].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[4].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[5].d);
    tmp += c;
    tmp += ">";
    path_map["D1"] = new string(tmp);
    if(DEBUG) cout << "D1=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[6].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[7].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[8].a);
    tmp += c;
    tmp += ">";
    path_map["A2"] = new string(tmp);
    if(DEBUG) cout << "A2=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[6].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[7].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[8].b);
    tmp += c;
    tmp += ">";
    path_map["B2"] = new string(tmp);
    if(DEBUG) cout << "B2=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[6].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[7].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[8].c);
    tmp += c;
    tmp += ">";
    path_map["C2"] = new string(tmp);
    if(DEBUG) cout << "C2=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[6].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[7].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -(*spl)[i]->pc_vec[8].d);
    tmp += c;
    tmp += ">";
    path_map["D2"] = new string(tmp);
    if(DEBUG) cout << "D2=" << tmp << endl;

    *pp += *fill_tpl(path_map, tpl);
  }

  *pp += "#end\n";

  return pp;
}


string* get_povray_light( vector< SplinePart*>* spl) {

  string tpl = "#range (!R1!, !R2!)\n";
  tpl += "\t!A! + !B! * (clock - !R1!) + "; 
  tpl += "!C!/2 * pow(clock - !R1!, 2) + ";
  tpl += "!D!/6 * pow(clock - !R1!, 3)\n"; 
  tpl +=  "#break\n";
  

  char c[FLT_SIZE];
  map<string, string*> path_map;
  string tmp;
  string* pp = new string;

  *pp = "#switch(clock)\n";

  for( int i = 0; i < spl->size()-1; i++) {
    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->t);    
    tmp += c;
    path_map["R1"] = new string(tmp);
    if(DEBUG) cout << "R1=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i+1]->t);    
    tmp += c;
    path_map["R2"] = new string(tmp);
    if(DEBUG) cout << "R2=" << tmp << endl;
    
    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].a + (*spl)[i]->pc_vec[6].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].a + (*spl)[i]->pc_vec[7].a);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -((*spl)[i]->pc_vec[2].a + (*spl)[i]->pc_vec[8].a));
    tmp += c;
    tmp += ">";
    path_map["A"] = new string(tmp);
    if(DEBUG) cout << "A=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].b + (*spl)[i]->pc_vec[6].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].b + (*spl)[i]->pc_vec[7].b);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -((*spl)[i]->pc_vec[2].b + (*spl)[i]->pc_vec[8].b));
    tmp += c;
    tmp += ">";
    path_map["B"] = new string(tmp);
    if(DEBUG) cout << "B=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].c + (*spl)[i]->pc_vec[6].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].c + (*spl)[i]->pc_vec[7].c);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -((*spl)[i]->pc_vec[2].c + (*spl)[i]->pc_vec[8].c));
    tmp += c;
    tmp += ">";
    path_map["C"] = new string(tmp);
    if(DEBUG) cout << "C=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[0].d + (*spl)[i]->pc_vec[6].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[1].d + (*spl)[i]->pc_vec[7].d);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", -((*spl)[i]->pc_vec[2].d + (*spl)[i]->pc_vec[8].d));
    tmp += c;
    tmp += ">";
    path_map["D"] = new string(tmp);
    if(DEBUG) cout << "D=" << tmp << endl;

    *pp += *fill_tpl(path_map, tpl);
  }

  *pp += "#end\n";

  return pp;
}


string* get_povray_x4( vector< SplinePart*>* spl) {
  string tpl = "#range (!R1!, !R2!)\n";
  tpl += "\t !A! + !B! * (clock - !R1!) + "; 
  tpl += "!C!/2 * pow(clock - !R1!, 2) + ";
  tpl += "!D!/6 * pow(clock - !R1!, 3)\n"; 
  tpl +=  "#break\n";
  

    
  char c[FLT_SIZE];
  map<string, string*> path_map;
  string tmp;
  string* pp = new string;

  *pp = "#switch(clock)\n";

  for( int i = 0; i < spl->size()-1; i++) {
    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->t);    
    tmp += c;
    path_map["R1"] = new string(tmp);
    if(DEBUG) cout << "R1=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i+1]->t);    
    tmp += c;
    path_map["R2"] = new string(tmp);
    if(DEBUG) cout << "R2=" << tmp << endl;
    
    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[9].a);
    tmp += c;
    path_map["A"] = new string(tmp);
    if(DEBUG) cout << "A=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[9].b);
    tmp += c;
    path_map["B"] = new string(tmp);
    if(DEBUG) cout << "B=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[9].c);
    tmp += c;
    path_map["C"] = new string(tmp);
    if(DEBUG) cout << "C=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", (*spl)[i]->pc_vec[9].d);
    tmp += c;
    path_map["D"] = new string(tmp);
    if(DEBUG) cout << "D=" << tmp << endl;

    *pp += *fill_tpl(path_map, tpl);
  }

  *pp += "#end\n";

  return pp;
}  


SceneSetupPage::SceneSetupPage(map<string, string*>& lookup, 
			       char* tplfile) : 
  vbox1(false,0),
  button1("fill it"),
  button2("write file"),
  checkbox1("edit .pov file")
{


  this->lookup = lookup;
  this->tplfile = tplfile;
  filled = new string("-- template file not yet processed --\n");
  
  pack1(vbox1, FALSE, FALSE);
  pack2(sw, TRUE, FALSE);
  sw.set_resize_mode(GTK_RESIZE_PARENT);

  vbox1.pack_start(hbox2, false, false, 0);

  hbox2.pack_start(button1, true, false, 0);
  hbox2.pack_start(button2, true, false, 0);
  hbox2.pack_start(checkbox1, true, false, 0);

  sw.set_border_width(5);
  sw.set_policy( GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  sw.add( txt);

  txt.set_editable(false);
  txt.insert( *filled);


  
  button1.clicked.connect( bind<int>(slot(this, &SceneSetupPage::on_button_clicked), 1));
  button2.clicked.connect( bind<int>(slot(this, &SceneSetupPage::on_button_clicked), 2));
  checkbox1.clicked.connect( slot(this, &SceneSetupPage::on_toggle));
  

}  



void SceneSetupPage::entry_activated(int id) {

  ostringstream buf;
  //buf << camera_Gtk_CoordEntry.get_coord();
  //string *buf1 = new string(buf.str());
  //lookup["LOCATION"] = buf1;

}	  

void SceneSetupPage::on_button_clicked(int id) {

  ifstream tplstream( tplfile);
  string *tpl = new string;
  char* pov_out = "data/q.pov";

  switch(id) {
  case 1:

    if( !tplstream) {
      cerr << "error: cannot open " << tplfile << endl;
      *tpl = "";
    } else {
      getline(tplstream, *tpl, (char) -1);
    }

    txt.backward_delete(filled->size());


    filled = fill_tpl( lookup, *tpl);
    txt.insert(*filled);
    txt.set_editable(true);
    break;
  case 2:
    
    ofstream to(pov_out);
    if( !to) {
      cerr << "error: could not open " << pov_out << endl;
      break;
    }
    
    to << txt.get_chars(0, -1);
    to.close();

  }

}	  

void SceneSetupPage::on_toggle() {
  if( checkbox1.get_active()) {
    sw.set_usize( 400, 400);
    sw.show();
  } else {
    sw.set_usize( 0, 0);
    sw.hide();
    set_position( vbox1.height());
  }
}	  

PovrayOptionsPage::PovrayOptionsPage(map<string, string*>& lookup, char* tplfile) : 
  vbox1(false,0),
  hbox1(false,0),
  button1("fill it"),
  button2("write file"),
  checkbox1("edit .ini file")
{

  list<char* > ll;
  GroupEntry* ge;

  ll.push_back( "dimensions");
  ll.push_back( "width");
  ll.push_back( "heigth");

  ge = manage( new GroupEntry(ll));
  ge->entry_activated.connect(bind<int >(slot( this, &PovrayOptionsPage::entry_activated), 0));

  /* some defaults */
  ge->set_entry(0, "320");
  ge->set_entry(1, "200");
  gentry_vec.push_back(ge);
  hbox1.pack_start(*ge);

  ll.clear();
  ll.push_back( "file options");
  ll.push_back( "filename");
  ll.push_back( "format");
  ll.push_back( "initial frame");
  ll.push_back( "final frame");

  ge = manage( new GroupEntry(ll, 80, 22, 20));
  ge->entry_activated.connect(bind<int >(slot( this, &PovrayOptionsPage::entry_activated), 1));
  /* some defaults */
  ge->set_entry(0, "q.pov");
  ge->set_entry(1, "TODO .tga");
  ge->set_entry(2, "1");
  ge->set_entry(3, "20");
  gentry_vec.push_back(ge);
  hbox1.pack_start(*ge);

  this->lookup = lookup;
  this->tplfile = tplfile;
  filled = new string("-- template file not yet processed --\n");


  pack1(vbox1, FALSE, FALSE);
  pack2(sw, TRUE, FALSE);

  vbox1.pack_start(hbox1, true, false, 0);
  vbox1.pack_start(hbox2, false, false, 0);


  hbox2.pack_start(button1, true, false, 0);
  hbox2.pack_start(button2, true, false, 0);
  hbox2.pack_start(checkbox1, true, false, 0);


  sw.set_border_width(5);
  sw.set_policy( GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  sw.add( txt);

  txt.set_editable(false);
  txt.insert( *filled);

 
  button1.clicked.connect( bind<int>(slot(this, &PovrayOptionsPage::on_button_clicked), 1));
  button2.clicked.connect( bind<int>(slot(this, &PovrayOptionsPage::on_button_clicked), 2));
  checkbox1.clicked.connect( slot(this, &PovrayOptionsPage::on_toggle));

}  


void PovrayOptionsPage::entry_activated(int entry_id, int frame_id) {
  cout << gentry_vec[frame_id]->get_entry(entry_id) << endl;
}	  

void PovrayOptionsPage::on_button_clicked(int id) {

  ifstream tplstream( tplfile);
  string *tpl = new string;
  char* ini_out = "data/q.ini";

  switch(id) {
  case 1:
    if( !tplstream) {
      cerr << "error: cannot open " << tplfile << endl;
      *tpl = "";
    } else {
      getline(tplstream, *tpl, (char) -1);
    }

    lookup["WIDTH"] = new string(gentry_vec[0]->get_entry(0));
    lookup["HEIGHT"] = new string(gentry_vec[0]->get_entry(1));

    lookup["POV_FILE"] = new string(gentry_vec[1]->get_entry(0));
    lookup["INIT_FRAME"] = new string(gentry_vec[1]->get_entry(2));
    lookup["FINAL_FRAME"] = new string(gentry_vec[1]->get_entry(3));

    txt.backward_delete(filled->size());
    filled = fill_tpl( lookup, *tpl);
    txt.insert(*filled);
    txt.set_editable(true);
    break;
  case 2:

    ofstream to(ini_out);
    if( !to) {
      cerr << "error: could not open " << ini_out << endl;
      break;
    }
    
    to << txt.get_chars(0, -1);
    to.close();

  }
}	  



void PovrayOptionsPage::on_toggle() {
  if( checkbox1.get_active()) {
    sw.set_usize( 400, 400);
    sw.show();
  } else {
    sw.set_usize( 0, 0);
    sw.hide();
  }
}	  


void PovrayOptionsPage::update_endtime(double time) {
  char c[FLT_SIZE];
  string tmp;

  tmp = "";
  snprintf( c, FLT_SIZE, "%f", time);    
  tmp += c;

  lookup["ENDTIME"] = new string(tmp);
}

 
PovrayOut::PovrayOut( map<string, string*>& lookup, 
		      char* pov_tplfile,
		      char* ini_tplfile) : 
  vbox3(false,0),
  popix("./pixmaps/big.FilmNegative.xpm"),
  ss_page( lookup, pov_tplfile), 
  po_page( lookup, ini_tplfile)
{
  main_button.add(popix);

  set_title("QuatFract Povray Output Generator");

  add(nb);

  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( ss_page, "Scene setup"));
  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( po_page, "Povray options"));
  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( vbox3, "Let's trace tha rays"));


  ss_page.on_toggle();
  po_page.on_toggle();

}

